from django.core.paginator import Paginator
from django.template import Library
from django.utils.translation import ugettext_lazy as _
from touchtechnology.news.models import Article, Category

register = Library()


@register.inclusion_tag('touchtechnology/news/_related_list.html',
                        takes_context=True)
def related_articles(context, article, limit=None, order_by=None):
    categories = article.categories.live()
    articles = Article.objects.live() \
                              .exclude(pk=article.pk) \
                              .filter(categories__in=categories) \
                              .distinct()

    if order_by is not None:
        articles = articles.order_by(*order_by.split(','))

    if limit is not None:
        articles = articles[:int(limit)]

    # FIXME backwards compatibility for custom templates
    context['slice'] = ':'

    context['article_list'] = articles
    return context


@register.inclusion_tag('touchtechnology/news/_related_list.html',
                        takes_context=True)
def related_categories(context, article=None, limit=None):
    """
    If an article is provided, then we select categories relating to it.
    Otherwise we select all article categories.
    """
    if article is None:
        categories = Category.objects.all()
    else:
        categories = article.categories.all()
    context['category_list'] = categories
    return context


@register.inclusion_tag('touchtechnology/news/_latest_articles.html',
                        takes_context=True)
def latest_articles(context, count=5, title=_("Latest News")):
    articles = Article.objects.live()
    paginator = Paginator(articles, count)
    page = paginator.page(1)
    context['paginator'] = paginator
    context['page'] = page
    context['article_list'] = page.object_list
    context['title'] = title
    return context



import unittest
from datetime import datetime

from django.test.utils import override_settings
from django.utils import timezone
from freezegun import freeze_time
from test_plus import TestCase

from tournamentcontrol.competition.tests import factories


@override_settings(ROOT_URLCONF='tournamentcontrol.competition.tests.urls')
class GoodViewTests(TestCase):

    def test_index(self):
        self.assertGoodView('competition:index')

    def test_competition(self):
        competition = factories.CompetitionFactory.create()
        self.assertGoodView('competition:competition', competition.slug)

    def test_season(self):
        season = factories.SeasonFactory.create()
        self.assertGoodView(
            'competition:season', season.competition.slug, season.slug)

    def test_season_calendar(self):
        # TODO load matches to see if the query count is scaled properly
        season = factories.SeasonFactory.create()
        self.assertGoodView(
            'competition:calendar', season.competition.slug, season.slug)

    def test_season_videos(self):
        season = factories.SeasonFactory.create()
        self.assertGoodView(
            'competition:season-videos', season.competition.slug, season.slug)

    def test_club(self):
        club = factories.ClubFactory.create()
        stage = factories.StageFactory.create()
        stage.division.season.competition.clubs.add(club)
        team = factories.TeamFactory.create(
            club=club, division=stage.division)
        factories.MatchFactory.create_batch(
            stage=stage, home_team=team, size=10)
        self.assertGoodView('competition:club',
                            competition=stage.division.season.competition.slug,
                            season=stage.division.season.slug,
                            club=club.slug)

    def test_club_calendar(self):
        club = factories.ClubFactory.create()
        stage = factories.StageFactory.create()
        stage.division.season.competition.clubs.add(club)
        team = factories.TeamFactory.create(
            club=club, division=stage.division)
        factories.MatchFactory.create_batch(
            stage=stage, home_team=team, size=10)
        self.assertGoodView('competition:calendar',
                            competition=stage.division.season.competition.slug,
                            season=stage.division.season.slug,
                            club=club.slug)

    def test_division(self):
        division = factories.DivisionFactory.create()
        self.assertGoodView('competition:division',
                            division.season.competition.slug,
                            division.season.slug,
                            division.slug)

    def test_division_calendar(self):
        division = factories.DivisionFactory.create()
        self.assertGoodView('competition:calendar',
                            competition=division.season.competition.slug,
                            season=division.season.slug,
                            division=division.slug)

    def test_stage(self):
        stage = factories.StageFactory.create()
        self.assertGoodView('competition:stage',
                            stage.division.season.competition.slug,
                            stage.division.season.slug,
                            stage.division.slug,
                            stage.slug)

    def test_stage_group(self):
        pool = factories.StageGroupFactory.create()
        self.assertGoodView('competition:pool',
                            pool.stage.division.season.competition.slug,
                            pool.stage.division.season.slug,
                            pool.stage.division.slug,
                            pool.stage.slug,
                            pool.slug)

    def test_match(self):
        match = factories.MatchFactory.create()
        self.assertGoodView('competition:match',
                            match.stage.division.season.competition.slug,
                            match.stage.division.season.slug,
                            match.stage.division.slug,
                            match.pk)

    def test_match_gone(self):
        opts = [
            {'is_bye': True},
            {'home_team': None},
            {'away_team': None},
        ]
        for kw in opts:
            match = factories.MatchFactory.create(**kw)
            self.get('competition:match',
                     match.stage.division.season.competition.slug,
                     match.stage.division.season.slug,
                     match.stage.division.slug,
                     match.pk)
            self.response_410()

    def test_match_video(self):
        match = factories.MatchFactory.create(videos=['https://youtu.be/jNQXAC9IVRw'])
        self.assertGoodView('competition:match-video',
                            match.stage.division.season.competition.slug,
                            match.stage.division.season.slug,
                            match.stage.division.slug,
                            match.pk)
        self.assertResponseContains(
            '<iframe width="480" height="360" '
            'src="http://www.youtube.com/embed/jNQXAC9IVRw?wmode=opaque" '
            'frameborder="0" allowfullscreen>'
            '</iframe>')

    def test_match_video_gone(self):
        opts = [
            {'is_bye': True},
            {'home_team': None},
            {'away_team': None},
        ]
        for kw in opts:
            match = factories.MatchFactory.create(**kw)
            self.get('competition:match-video',
                     match.stage.division.season.competition.slug,
                     match.stage.division.season.slug,
                     match.stage.division.slug,
                     match.pk)
            self.response_410()


@override_settings(ROOT_URLCONF='tournamentcontrol.competition.tests.urls')
class FrontEndTests(TestCase):

    def test_competition_list(self):
        comp_1 = factories.CompetitionFactory.create()
        comp_2 = factories.CompetitionFactory.create(enabled=False)
        self.get('competition:index')
        self.assertResponseContains(comp_1.title, html=False)
        self.assertResponseNotContains(comp_2.title, html=False)

    def test_season_list(self):
        season = factories.SeasonFactory.create()
        self.assertGoodView(
            'competition:season', season.competition.slug, season.slug)
        self.assertResponseContains(season.title, html=False)

        season = factories.SeasonFactory.create(competition__enabled=False)
        self.get('competition:competition', season.competition.slug)
        self.response_404()

    def test_division_list(self):
        division = factories.DivisionFactory.create()
        self.assertGoodView('competition:season',
                            division.season.competition.slug,
                            division.season.slug)
        self.assertResponseContains(division.title, html=False)

    def test_division_view(self):
        division = factories.DivisionFactory.create()
        self.assertGoodView('competition:division',
                            division.season.competition.slug,
                            division.season.slug,
                            division.slug)
        self.assertResponseContains(division.title, html=False)

    @freeze_time('2013-11-01')
    def test_upcoming_matches(self):
        division = factories.DivisionFactory.create()
        factories.MatchFactory.create_batch(
            stage__division=division,
            datetime=datetime(2013, 11, 22, 10, tzinfo=timezone.utc),
            size=10)
        self.assertGoodView('competition:season',
                            division.season.competition.slug,
                            division.season.slug)
        self.assertResponseContains('Nov. 22, 2013', html=False)

    def test_division_match_list(self):
        division = factories.DivisionFactory.create()
        factories.MatchFactory.create_batch(stage__division=division, size=10)
        self.assertGoodView('competition:division',
                            division.season.competition.slug,
                            division.season.slug,
                            division.slug)
        for team in division.teams.all():
            href = self.reverse('competition:team',
                                division.season.competition.slug,
                                division.season.slug,
                                division.slug,
                                team.slug)
            self.assertResponseContains(
                '<a href="{0}">{1}</a>'.format(href, team.title))

    def test_team_calendar(self):
        team = factories.TeamFactory.create()
        factories.MatchFactory.create_batch(stage__division=team.division,
                                            home_team=team,
                                            size=5)
        self.assertGoodView('competition:calendar',
                            team.division.season.competition.slug,
                            team.division.season.slug,
                            team.division.slug,
                            team.slug)
        for opponent in team.division.teams.exclude(pk=team.pk):
            subject = '{} vs {}'.format(team.title, opponent.title)
            self.assertResponseContains(subject, html=False)

    @unittest.expectedFailure
    def test_team_calendar_disabled(self):
        season = factories.SeasonFactory.create(disable_calendar=True)
        team = factories.TeamFactory.create(division__season=season)
        factories.MatchFactory.create_batch(stage__division=team.division,
                                            home_team=team,
                                            size=5)
        self.assertGoodView('competition:calendar',
                            team.division.season.competition.slug,
                            team.division.season.slug,
                            team.division.slug,
                            team.slug)

    def test_division_calendar(self):
        team = factories.TeamFactory.create()
        factories.MatchFactory.create_batch(stage__division=team.division,
                                            home_team=team,
                                            size=5)
        self.assertGoodView('competition:calendar',
                            team.division.season.competition.slug,
                            team.division.season.slug,
                            team.division.slug)
        for opponent in team.division.teams.exclude(pk=team.pk):
            subject = '{} vs {}'.format(team.title, opponent.title)
            self.assertResponseContains(subject, html=False)

    @unittest.expectedFailure
    def test_division_calendar_disabled(self):
        season = factories.SeasonFactory.create(disable_calendar=True)
        team = factories.TeamFactory.create(division__season=season)
        factories.MatchFactory.create_batch(stage__division=team.division,
                                            home_team=team,
                                            size=5)
        self.assertGoodView('competition:calendar',
                            team.division.season.competition.slug,
                            team.division.season.slug,
                            team.division.slug)

    def test_season_calendar(self):
        team = factories.TeamFactory.create()
        factories.MatchFactory.create_batch(stage__division=team.division,
                                            home_team=team,
                                            size=5)
        self.assertGoodView('competition:calendar',
                            team.division.season.competition.slug,
                            team.division.season.slug)
        for opponent in team.division.teams.exclude(pk=team.pk):
            subject = '{} vs {}'.format(team.title, opponent.title)
            self.assertResponseContains(subject, html=False)

    @unittest.expectedFailure
    def test_season_calendar_disabled(self):
        season = factories.SeasonFactory.create(disable_calendar=True)
        team = factories.TeamFactory.create(division__season=season)
        factories.MatchFactory.create_batch(stage__division=team.division,
                                            home_team=team,
                                            size=5)
        self.assertGoodView('competition:calendar',
                            team.division.season.competition.slug,
                            team.division.season.slug)



from tournamentcontrol.competition.signals.decorators import (
    disable_for_loaddata,
)


@disable_for_loaddata
def set_ground_latlng(sender, instance, created, *args, **kwargs):
    if created and not instance.latlng:
        instance.latlng = instance.venue.latlng
        instance.save()


@disable_for_loaddata
def set_ground_timezone(sender, instance, created, *args, **kwargs):
    if created and not instance.timezone:
        instance.timezone = instance.venue.timezone
        instance.save()



from django.conf import settings
from first import first

__all__ = (
    'NODE_CACHE',
    'PAGE_CONTENT_BLOCKS',
    'PAGE_CONTENT_CLASSES',
    'PAGE_TEMPLATE_BASE',
    'PAGE_TEMPLATE_FOLDER',
    'PAGE_TEMPLATE_REGEX',
)

S = lambda n, d=None: getattr(settings, 'TOUCHTECHNOLOGY_' + n, d)

project_template_dirs = first(getattr(settings, 'TEMPLATES', ()), {}).get('DIRS', [])
project_template_base = first(project_template_dirs, 'templates')

NODE_CACHE = getattr(settings, 'TOUCHTECHNOLOGY_NODE_CACHE', 'default')

# These settings allow us to determine where page templates live, which allows
# us not to need to specify each actual template in code.

PAGE_CONTENT_BLOCKS = S('PAGE_CONTENT_BLOCKS', 1)
PAGE_CONTENT_CLASSES = S('PAGE_CONTENT_CLASSES', ('copy',))
PAGE_TEMPLATE_BASE = S('PAGE_TEMPLATE_BASE', project_template_base)
PAGE_TEMPLATE_FOLDER = S('PAGE_TEMPLATE_FOLDER',
                         'touchtechnology/content/pages/')
PAGE_TEMPLATE_REGEX = S('PAGE_TEMPLATE_REGEX', r'\.html$')

TENANT_MEDIA_PUBLIC = S('TENANT_MEDIA_PUBLIC', True)



from datetime import date, datetime

import pytz
import six
from django.utils import timezone
from freezegun import freeze_time
from test_plus import TestCase
from tournamentcontrol.competition.dashboard import matches_require_basic_results
from tournamentcontrol.competition.tests.factories import MatchFactory


class BasicResultTests(TestCase):
    @freeze_time("2019-07-15 09:00 UTC")
    def test_bare_all_in_utc(self):
        match = MatchFactory.create(
            datetime=datetime(2019, 7, 15, 8, 30, tzinfo=timezone.utc),
            stage__division__season__timezone="UTC",
        )
        six.assertCountEqual(self, matches_require_basic_results(), [match])

    @freeze_time("2019-07-15 09:00 +13")
    def test_bare_match_in_samoa(self):
        tzname = "Pacific/Apia"
        tzinfo = pytz.timezone(tzname)
        match = MatchFactory.create(
            datetime=timezone.make_aware(datetime(2019, 7, 15, 8, 30), tzinfo),
            stage__division__season__timezone=tzname,
        )
        six.assertCountEqual(self, matches_require_basic_results(), [match])

    @freeze_time("2019-07-15 09:00 +13")
    def test_bare_match_in_samoa_bye(self):
        tzname = "Pacific/Apia"
        tzinfo = pytz.timezone(tzname)
        match = MatchFactory.create(
            round=1,
            is_bye=True,
            away_team=None,
            datetime=None,
            date=date(2019, 7, 15),
            time=None,
            stage__division__season__timezone=tzname,
        )
        # A bye's would never exist in isolation.
        MatchFactory.create(
            round=1,
            datetime=timezone.make_aware(datetime(2019, 7, 15, 8, 30), tzinfo),
            stage=match.stage,
            home_team_score=5,
            away_team_score=4,
        )
        six.assertCountEqual(self, matches_require_basic_results(), [match])



from django.conf.urls import include, url
from tournamentcontrol.competition.rest.v1._routers import *

urlpatterns = [
    url(r"^", include(router.urls)),
    url(r"^", include(competition_router.urls)),
    url(r"^", include(season_router.urls)),
    url(r"^", include(division_router.urls)),
]



from django.conf.urls import url
from touchtechnology.common.sites import AccountsSite
from tournamentcontrol.competition.sites import competition

accounts = AccountsSite()

urlpatterns = [
    url(r'^accounts/', accounts.urls),
    url(r'^', competition.urls),
]



from __future__ import unicode_literals

import base64
import functools
import operator
from datetime import timedelta
from operator import or_

import pytz
from dateutil.relativedelta import relativedelta
from django.conf import settings
from django.conf.urls import include, url
from django.contrib import messages
from django.contrib.sitemaps import views as sitemaps_views
from django.core.exceptions import ObjectDoesNotExist
from django.db.models import Case, Count, F, Q, Sum, When
from django.http import Http404, HttpResponse, HttpResponseGone
from django.shortcuts import get_object_or_404
from django.utils import timezone
from django.utils.encoding import smart_bytes
from django.utils.module_loading import import_string
from django.utils.translation import ugettext, ugettext_lazy as _
from icalendar import Calendar, Event
from six.moves import reduce
from touchtechnology.common.decorators import login_required_m
from touchtechnology.common.sites import Application
from touchtechnology.common.utils import get_403_or_None, get_perms_for_model
from tournamentcontrol.competition import rank
from tournamentcontrol.competition.dashboard import (
    matches_require_basic_results, matches_require_details_results,
)
from tournamentcontrol.competition.decorators import competition_slug
from tournamentcontrol.competition.forms import (
    ConfigurationForm, MatchResultFormSet, MatchStatisticFormset, MultiConfigurationForm,
    RankingConfigurationForm,
)
from tournamentcontrol.competition.models import (
    Competition, Match, Person, SimpleScoreMatchStatistic,
)
from tournamentcontrol.competition.utils import FauxQueryset, team_needs_progressing


def permissions_required(request, model, instance=None, return_403=True,
                         accept_global_perms=True, create=False, perms=None):
    # If no perms are specified, build sensible default using built in
    # permission types that come batteries included with Django.
    if perms is None:
        perms = get_perms_for_model(model, change=True)

        # When we're doing a creation we should have permission to create the object.
        if create:
            perms = get_perms_for_model(model, add=True)

    # Determine the user's permission to edit this object using the
    # get_403_or_None - saves decorating view method with
    # @permission_required_or_403
    has_permission = get_403_or_None(
        request, perms, obj=instance, return_403=return_403,
        accept_global_perms=accept_global_perms)

    if has_permission is not None:
        return has_permission


class CompetitionAdminMixin(object):
    """
    Some administrative functions should be allowed to authenticated users on
    the front-end of the site. As long as we only use primitives from the base
    ``touchtechnology.common.sites.Application`` class it will integrate fine
    with both.
    """
    def day_runsheet(self, request, season, date, extra_context, templates=None, **kwargs):
        matches = season.matches.filter(date=date).order_by('is_bye', 'datetime', 'play_at')
        templates = self.template_path('runsheet.html', season.slug, season.competition.slug)
        return self.generic_list(request, matches,
                                 templates=templates,
                                 paginate_by=0,
                                 permission_required=False,
                                 extra_context=extra_context)

    def match_results(self, request, competition, season, date, extra_context, redirect_to,
                      division=None, time=None, **kwargs):
        matches = Match.objects.filter(
            stage__division__season=season,
            stage__division__season__competition=competition,
            date=date,
        )

        if division is not None:
            matches = matches.filter(stage__division=division)

        # ensure only matches with progressed teams are able to be updated
        matches = matches.exclude(team_needs_progressing, is_bye=False)
        matches = matches.exclude(home_team__isnull=True, away_team__isnull=True)

        # FIXME too complex, be verbose so we can all read and understand it
        if time is not None:
            base = Q(time=time)
            bye_kwargs = matches.filter(time=time).values('stage', 'round')
            time_or_byes = functools.reduce(
                operator.or_,
                [Q(time__isnull=True, **kw) for kw in bye_kwargs],
                base)
            matches = matches.filter(time_or_byes)
        else:
            matches = matches.order_by('date', 'time', 'play_at')

        match_queryset = matches.filter(is_bye=False)
        bye_queryset = matches.filter(is_bye=True)

        if request.method == 'POST':
            match_formset = MatchResultFormSet(
                data=request.POST, queryset=match_queryset, prefix='matches')
            bye_formset = MatchResultFormSet(
                data=request.POST, queryset=bye_queryset, prefix='byes')

            if match_formset.is_valid() and bye_formset.is_valid():
                match_formset.save()
                bye_formset.save()
                messages.success(request, _("Your changes have been saved."))
                return self.redirect(redirect_to)
        else:
            match_formset = MatchResultFormSet(queryset=match_queryset, prefix='matches')
            bye_formset = MatchResultFormSet(queryset=bye_queryset, prefix='byes')

        context = {
            'competition': competition,
            'season': season,
            'date': date,
            'division': division,
            'match_formset': match_formset,
            'bye_formset': bye_formset,
            'matches': matches,
            'cancel_url': redirect_to,
        }
        context.update(extra_context)

        templates = self.template_path('match_results.html')
        return self.render(request, templates, context)

    def edit_match_detail(self, request, stage, match, extra_context, redirect_to, **kwargs):
        conditions = {
            'home_team__isnull': False,
            'away_team__isnull': False,
            'home_team_score__isnull': False,
            'away_team_score__isnull': False,
        }

        if match is None:
            match = get_object_or_404(stage.matches, pk=match.pk, **conditions)

        def team_faux_queryset(team):
            stats = FauxQueryset(SimpleScoreMatchStatistic, team=team)
            for player in team.people.filter(is_player=True):
                try:
                    statistic = SimpleScoreMatchStatistic.objects.get(
                        match=match,
                        player=player.person)
                except ObjectDoesNotExist:
                    statistic = SimpleScoreMatchStatistic(
                        match=match,
                        player=player.person,
                        number=player.number,
                        played=1)
                stats.append(statistic)
            return stats

        home_queryset = team_faux_queryset(match.home_team)
        away_queryset = team_faux_queryset(match.away_team)

        if request.method == 'POST':
            home = MatchStatisticFormset(data=request.POST,
                                         score=match.home_team_score,
                                         prefix='home',
                                         queryset=home_queryset)

            away = MatchStatisticFormset(data=request.POST,
                                         score=match.away_team_score,
                                         prefix='away',
                                         queryset=away_queryset)

            if home.is_valid() and away.is_valid():
                home.save()
                away.save()
                messages.success(request, _("Your changes have been saved."))
                return self.redirect(redirect_to)

        else:
            home = MatchStatisticFormset(prefix='home',
                                         score=match.home_team_score,
                                         queryset=home_queryset)
            away = MatchStatisticFormset(prefix='away',
                                         score=match.away_team_score,
                                         queryset=away_queryset)

        context = {
            'object': match,
            'formsets': (home, away),
        }
        context.update(extra_context)

        templates = self.template_path('match_detail.html')
        return self.render(request, templates, context)


class CompetitionSite(CompetitionAdminMixin, Application):

    kwargs_form_class = ConfigurationForm

    def __init__(self, name='competition', app_name='competition', **kwargs):
        # store the node for future reference
        self.node = kwargs.get('node')
        super(CompetitionSite, self).__init__(
            name=name, app_name=app_name, **kwargs)
        self._competitions = Competition.objects.filter(enabled=True)
        if 'competition' in kwargs:
            self._competitions = self._competitions.filter(
                slug=kwargs['competition'])

    def result_urls(self):
        return [
            url(r'^$', self.results, name='results'),
            url(r'^match/(?P<match>\d+)/$', self.edit_match_detail, name='match-details'),
            url(r'^(?P<datestr>\d{8})/$', self.results, name='results'),
            url(r'^(?P<datestr>\d{8})/(?P<timestr>\d{4})/$', self.match_results, name='match-results'),
        ]

    def runsheet_urls(self):
        return [
            url(r'^$', self.runsheet, name='runsheet'),
            url(r'^(?P<datestr>\d{8})/$', self.day_runsheet, name='runsheet'),
        ]

    def season_urls(self):
        return [
            url(r'^$', self.season, name='season'),
            url(r'^forfeit/$', self.forfeit_list, name='forfeit-list'),
            url(r'^forfeit/(?P<match>\d+)/$', self.forfeit, name='forfeit'),
            url(r'^videos/$', self.season_videos, name='season-videos'),
            url(r'^club:(?P<club>[\w-]+)/$', self.club, name='club'),
            url(r'^club:(?P<club>[\w-]+).ics$', self.calendar, name='calendar'),
            url(r'^results/', include(self.result_urls())),
            url(r'^runsheet/', include(self.runsheet_urls())),
            url(r'^(?P<division>[\w-]+).ics$', self.calendar, name='calendar'),
            url(r'^(?P<division>[\w-]+)/$', self.division, name='division'),
            url(r'^(?P<division>[\w-]+):(?P<stage>[\w-]+)/$', self.stage, name='stage'),
            url(r'^(?P<division>[\w-]+):(?P<stage>[\w-]+):(?P<pool>[\w-]+)/$', self.pool, name='pool'),
            url(r'^(?P<division>[\w-]+)/match:(?P<match>\d+)/$', self.match, name='match'),
            url(r'^(?P<division>[\w-]+)/match:(?P<match>\d+)/video/$', self.match_video, name='match-video'),
            url(r'^(?P<division>[\w-]+)/(?P<team>[\w-]+).ics$', self.calendar, name='calendar'),
            url(r'^(?P<division>[\w-]+)/(?P<team>[\w-]+)/$', self.team, name='team'),
        ]

    def competition_urls(self):
        return [
            url(r'^$', self.competition, name='competition'),
            url(r'^(?P<season>[\w-]+).ics$', self.calendar, name='calendar'),
            url(r'^(?P<season>[\w-]+)/', include(self.season_urls())),
        ]

    def get_urls(self):
        if 'season' in self.kwargs:
            urlpatterns = [
                url(r'^', include(self.season_urls()),
                    kwargs=self.kwargs),
            ]
        elif 'competition' in self.kwargs:
            urlpatterns = [
                url(r'^', include(self.competition_urls()),
                    kwargs=self.kwargs),
            ]
        else:
            urlpatterns = [
                url(r'^$', self.index, name='index'),
                url(r'^(?P<competition>[\w-]+)/',
                    include(self.competition_urls())),
            ]
        return urlpatterns

    def sitemap_index(self, request, node=None, competition=None, season=None, division=None, *args, **kwargs):
        # We need to "pop" the node keyword argument
        return sitemaps_views.index(request, *args, **kwargs)

    def sitemap_section(self, request, node=None, competition=None, season=None, division=None, *args, **kwargs):
        # We need to "pop" the node keyword argument
        return sitemaps_views.sitemap(request, *args, **kwargs)

    @property
    def urls(self):
        return self.get_urls(), self.app_name, self.name

    @property
    def competitions(self):
        return self._competitions.prefetch_related('seasons')

    def index(self, request, **kwargs):
        return self.generic_list(request, self._competitions,
                                 templates=self.template_path('index.html'),
                                 paginate_by=self._competitions.count(),
                                 extra_context=kwargs)

    @competition_slug
    def competition(self, request, competition, extra_context, **kwargs):
        templates = self.template_path('competition.html', competition.slug)
        return self.generic_detail(request, self._competitions,
                                   slug=competition.slug,
                                   templates=templates,
                                   extra_context=extra_context)

    @competition_slug
    def season(self, request, competition, season, extra_context, **kwargs):
        templates = self.template_path(
            'season.html', competition.slug, season.slug)
        extra_context.update(season.matches.exclude(is_bye=True).aggregate(
            timeslot_count=Count('datetime', distinct=True),
            match_count=Count('pk', distinct=True),
            points_scored=Sum(
                Case(
                    When(
                        statistics__match__stage__division__season=season,
                        then=F('statistics__points')))),
            caps_awarded=Sum(
                Case(
                    When(
                        statistics__match__stage__division__season=season,
                        then=F('statistics__played')))),
        ))
        return self.generic_detail(request, competition.seasons,
                                   slug=season.slug,
                                   templates=templates,
                                   extra_context=extra_context)

    @competition_slug
    def runsheet(self, request, competition, season, extra_context, **kwargs):
        context = {
            'dates': season.matches.dates('date', 'day')
        }
        context.update(extra_context)
        templates = self.template_path('runsheet_list.html', competition.slug, season.slug)
        return self.render(request, templates, context)

    @competition_slug
    def day_runsheet(self, request, season, date, extra_context, **kwargs):
        return super(CompetitionSite, self).day_runsheet(
            request, season, date, extra_context, **kwargs)

    @competition_slug
    @login_required_m
    def results(self, request, competition, season, extra_context, date=None, **kwargs):
        has_permission = permissions_required(request, Match, return_403=False)
        if has_permission is not None:
            return has_permission

        if date is not None:
            matches = season.matches.filter(date=date)
        else:
            matches = season.matches

        # Allow super-user accounts visibility to look 1 year into the future.
        if request.user.is_superuser:
            now = timezone.now() + relativedelta(years=1)
        else:
            now = None

        # Filter the list of matches to those which require result entry
        basic_results = matches_require_basic_results(now, matches)
        details_results = matches_require_details_results(matches, True)

        context = {
            "datetimes": [
                timezone.localtime(each, pytz.timezone(season.timezone))
                for each in basic_results.datetimes("datetime", "minute")
            ],
            "details": details_results,
        }
        context.update(extra_context)
        templates = self.template_path("results.html", competition.slug, season.slug)
        return self.render(request, templates, context)

    @competition_slug
    @login_required_m
    def match_results(self, request, competition, season, date, time, extra_context, **kwargs):
        has_permission = permissions_required(request, Match, return_403=False)
        if has_permission is not None:
            return has_permission
        redirect_to = self.reverse('results', args=(competition.slug, season.slug))
        return super(CompetitionSite, self).match_results(
            request, competition=competition, season=season,
            date=date, time=time,
            extra_context=extra_context,
            redirect_to=redirect_to,
            **kwargs)

    @login_required_m
    @competition_slug
    def edit_match_detail(self, request, match, extra_context, **kwargs):
        has_permission = permissions_required(request, Match, instance=match, return_403=False)
        if has_permission is not None:
            return has_permission
        redirect_to = self.reverse(
            'results',
            args=(match.stage.division.season.competition.slug, match.stage.division.season.slug))
        return super(CompetitionSite, self).edit_match_detail(
            request, stage=match.stage, match=match, extra_context=extra_context,
            redirect_to=redirect_to, **kwargs)

    @competition_slug
    def season_videos(self, request, competition, season, extra_context, **kwargs):
        templates = self.template_path(
            'season_videos.html', competition.slug, season.slug)
        queryset = (
            Match.objects
            .select_related(
                "stage__division",
                "home_team__club",
                "away_team__club",
                "play_at",
            )
            .exclude(videos__isnull=True)
            .filter(stage__division__season=season)
            .order_by("datetime", "play_at")
        )
        return self.generic_list(request, queryset,
                                 paginate_by=1000,
                                 templates=templates,
                                 extra_context=extra_context)

    @competition_slug
    def club(self, request, competition, season, club, extra_context,
             **kwargs):
        templates = self.template_path(
            'club.html', competition.slug, season.slug, club.slug)
        return self.generic_detail(request, season.clubs,
                                   slug=club.slug,
                                   templates=templates,
                                   extra_context=extra_context)

    @competition_slug
    def division(self, request, competition, season, division, extra_context,
                 **kwargs):
        templates = self.template_path(
            'division.html', competition.slug, season.slug, division.slug)
        extra_context.update(division.matches.exclude(is_bye=True).aggregate(
            timeslot_count=Count('datetime', distinct=True),
            match_count=Count('pk', distinct=True),
            points_scored=Sum(
                Case(
                    When(
                        statistics__match__stage__division=division,
                        then=F('statistics__points')))),
            caps_awarded=Sum(
                Case(
                    When(
                        statistics__match__stage__division=division,
                        then=F('statistics__played')))),
        ))
        return self.generic_detail(request, season.divisions,
                                   slug=division.slug,
                                   templates=templates,
                                   extra_context=extra_context)

    @competition_slug
    def stage(self, request, competition, season, division, stage,
              extra_context, **kwargs):
        templates = self.template_path(
            'stage.html',
            competition.slug, season.slug, division.slug, stage.slug)
        extra_context['parent'] = stage
        extra_context.update(stage.matches.exclude(is_bye=True).aggregate(
            timeslot_count=Count('datetime', distinct=True),
            match_count=Count('pk', distinct=True),
            points_scored=Sum(
                Case(
                    When(
                        statistics__match__stage=stage,
                        then=F('statistics__points')))),
            caps_awarded=Sum(
                Case(
                    When(
                        statistics__match__stage=stage,
                        then=F('statistics__played')))),
        ))
        return self.generic_detail(request, division.stages,
                                   slug=stage.slug,
                                   templates=templates,
                                   extra_context=extra_context)

    @competition_slug
    def pool(self, request, competition, season, division, stage, pool,
             extra_context, **kwargs):
        # FIXME sadly the template name was not changed when we refactored
        #       pools to be subordinates of Stage.
        templates = self.template_path(
            'divisiongroup.html', competition.slug, season.slug, division.slug,
            stage.slug, pool.slug)
        extra_context.update(pool.matches.exclude(is_bye=True).aggregate(
            timeslot_count=Count('datetime', distinct=True),
            match_count=Count('pk', distinct=True),
            points_scored=Sum(
                Case(
                    When(
                        statistics__match__stage_group=pool,
                        then=F('statistics__points')))),
            caps_awarded=Sum(
                Case(
                    When(
                        statistics__match__stage_group=pool,
                        then=F('statistics__played')))),
        ))
        return self.generic_detail(request, stage.pools,
                                   slug=pool.slug,
                                   templates=templates,
                                   extra_context=extra_context)

    @competition_slug
    def team(self, request, competition, season, division, team, extra_context,
             **kwargs):
        templates = self.template_path(
            'team.html', competition.slug, season.slug, division.slug,
            team.slug)

        extra_context.update(team.matches.exclude(is_bye=True).aggregate(
            timeslot_count=Count('datetime', distinct=True),
            match_count=Count('pk', distinct=True),
            points_scored=Sum(
                Case(
                    When(
                        statistics__player__teamassociation__team=team,
                        then=F('statistics__points')))),
            caps_awarded=Sum(
                Case(
                    When(
                        statistics__player__teamassociation__team=team,
                        then=F('statistics__played')))),
        ))
        return self.generic_detail(request, division.teams,
                                   slug=team.slug,
                                   templates=templates,
                                   extra_context=extra_context)

    @competition_slug
    def calendar(self, request, season, club=None, division=None, team=None, **kwargs):
        if season.disable_calendar:
            # The GONE response informs client that they should remove this resource
            # from their cache. When a calendar has been added to user's mobile device
            # they may never look at it again, but we continue to process the requests
            # which can have poor performance. Try to influence a cleanup of clients.
            return HttpResponseGone()

        if team is not None:
            matches = team.matches
        elif division is not None:
            matches = division.matches
        elif club is not None:
            matches = club.matches.filter(stage__division__season=season)
        else:
            matches = season.matches

        # Do not include matches which have not had the time scheduled
        matches = matches.exclude(datetime__isnull=True)

        # Perform select_related to reduce extra queries
        matches = matches.select_related('stage__division__season__competition')

        # Reduce the size of the data set to return from the database
        matches = matches.defer('stage__division__season__competition__copy')

        # Remove any matches that are part of a draft division unless being viewed
        # by a superuser.
        if not request.user.is_superuser:
            matches = matches.exclude(stage__division__draft=True)

        # For development server turn back plain text to make debugging easier
        if settings.DEBUG:
            content_type = 'text/plain'
        else:
            content_type = 'text/calendar'

        response = HttpResponse(content_type=content_type)

        cal = Calendar()
        cal.add('prodid', '-//Tournament Control//%s//' % request.get_host())
        cal.add('version', '2.0')

        for match in matches.order_by('datetime', 'play_at'):
            # Determine the resource path to the detailed match view
            path = self.reverse('match', kwargs={
                'match': match.pk,
                'division': match.stage.division.slug,
                'season': match.stage.division.season.slug,
                'competition': match.stage.division.season.competition.slug,
            })

            # Combine the resource path with our current request context
            url = request.build_absolute_uri(str(path))

            event = Event()
            event['uid'] = '%s@%s' % (
                base64.b64encode(smart_bytes(url)), request.get_host())
            event.add('summary', match.title)
            event.add('location', '{0} ({1})'.format(
                match.stage.division.title, match.stage.title))
            event.add('description', url)
            event.add('dtstart', match.datetime)
            # FIXME match duration should not be hardcoded
            event.add('dtend', match.datetime + timedelta(minutes=45))
            # FIXME should be the last modified time of the match
            event.add('dtstamp', timezone.now())

            cal.add_component(event)

        response.write(cal.to_ical())
        return response

    @competition_slug
    def match(self, request, competition, season, division, match,
              extra_context, **kwargs):
        if match.is_bye or match.home_team is None or match.away_team is None:
            # Temporary, realistically nobody should ever have this URL but we
            # will use GONE so that Google removes it from their index.
            return HttpResponseGone()
        templates = self.template_path(
            'match.html', competition.slug, season.slug, division.slug)
        return self.generic_detail(request, division.matches,
                                   pk=match.pk,
                                   templates=templates,
                                   extra_context=extra_context)

    @competition_slug
    def match_video(self, request, competition, season, division, match,
                    extra_context, **kwargs):
        if match.is_bye or match.home_team is None or match.away_team is None:
            # Temporary, realistically nobody should ever have this URL but we
            # will use GONE so that Google removes it from their index.
            return HttpResponseGone()
        templates = self.template_path(
            'videos.html', competition.slug, season.slug, division.slug)
        return self.generic_detail(request, division.matches,
                                   pk=match.pk,
                                   templates=templates,
                                   extra_context=extra_context)

        # TODO add related model to record videos
        # return self.generic_list(request, match.videos,
        #                          templates=templates,
        #                          extra_context=extra_context)

    @login_required_m
    @competition_slug
    def forfeit_list(self, request, competition, season, extra_context,
                     **kwargs):
        """
        List the matches that the visitor is permitted to forfeit. Must only
        show matches for teams that they are a registered member of (either as
        a player or other role).
        """
        try:
            teams = request.user.person.teams
        except Person.DoesNotExist:
            raise Http404

        matches = season.matches.filter(
            Q(home_team=teams) | Q(away_team=teams))

        context = {
            'matches': matches,
        }
        context.update(extra_context)

        templates = self.template_path('forfeit_list.html')
        return self.render(request, templates, context)

    @login_required_m
    @competition_slug
    def forfeit(self, request, competition, season, extra_context, match,
                **kwargs):
        """
        View that will allow a team member to forfeit a match they are due to
        be playing in.
        """
        try:
            teams = request.user.person.teams
        except Person.DoesNotExist:
            raise Http404

        matches = season.matches.filter(
            Q(home_team=teams) | Q(away_team=teams))

        redirect = self.redirect(self.reverse('forfeit-list', kwargs={
            'competition': competition.slug,
            'season': season.slug,
        }))

        if match not in matches:
            messages.add_message(
                request, messages.ERROR,
                "Attempting to forfeit a match you are not involved in.")
            return redirect

        # decide which team to forfeit
        if match.home_team in teams and match.away_team in teams:
            messages.add_message(
                request, messages.WARNING,
                _("You are a member of both teams for this match, please "
                  "contact the competition manager."))
            return redirect

        elif match.home_team in teams:
            team = match.home_team
        elif match.away_team in teams:
            team = match.away_team
        else:
            messages.add_message(
                request, messages.ERROR,
                _("You are not a member of either team, please contact the "
                  "competition manager."))
            return redirect

        if request.method == 'POST':
            # use the forfeit API to advise who made lodged the forfeit
            UNABLE_TO_POST = _("Unable to post forfeit, please contact the "
                               "competition manager.")
            try:
                success = match.forfeit(team, request.user.person)
            except AssertionError:
                # logger.exception("Failed precondition checks")
                messages.add_message(request, messages.ERROR, UNABLE_TO_POST)

            if success:
                messages.add_message(
                    request, messages.INFO,
                    _("The match has been forfeit, notifications will be sent "
                      "to affected parties shortly."))
            else:
                messages.add_message(request, messages.ERROR, UNABLE_TO_POST)

            return redirect

        context = {
            'match': match,
            'team': team,
        }
        context.update(extra_context)

        templates = self.template_path('forfeit.html')
        return self.render(request, templates, context)


class MultiCompetitionSite(CompetitionSite):

    kwargs_form_class = MultiConfigurationForm

    @classmethod
    def verbose_name(cls):
        return ugettext("Multiple Competitions")

    def __init__(self, name='competition', app_name='competition', **kwargs):
        self.node = kwargs.get('node')  # store the node for future reference
        super(CompetitionSite, self).__init__(name=name, app_name=app_name, **kwargs)
        self._competitions = Competition.objects.filter(enabled=True)
        if 'competition' in kwargs:
            q = functools.reduce(or_, [Q(slug=slug) for slug in kwargs['competition']])
            self._competitions = self._competitions.filter(q)

    def get_urls(self):
        urlpatterns = [
            url(r'^$', self.index, name='index'),
            url(r'^(?P<competition>[^/]+)/', include(self.competition_urls())),
        ]
        return urlpatterns


class RegistrationSite(Application):

    def __init__(self, name='rego', app_name='rego', **kwargs):
        super(RegistrationSite, self).__init__(
            name=name, app_name=app_name, **kwargs)

    def get_urls(self):
        return []


class TournamentCalculatorSite(Application):

    def __init__(self, name='calculator', app_name='calculator', **kwargs):
        self.form_class = import_string(kwargs.pop(
            'form_class',
            'tournamentcontrol.competition.forms.TournamentScheduleForm'))
        super(TournamentCalculatorSite, self).__init__(
            name=name, app_name=app_name, **kwargs)

    def get_urls(self):
        live_form_class = import_string('tournamentcontrol.competition.forms.'
                                        'DivisionTournamentScheduleForm')
        urlpatterns = [
            url(r'^$', self.index,
                name='index',
                kwargs={
                    'form_class': self.form_class,
                }),
            url(r'^live/$', self.index,
                name='division',
                kwargs={
                    'form_class': live_form_class,
                    'template_name': 'live.html',
                }),
        ]
        return urlpatterns

    def index(self, request, form_class, template_name='index.html',
              **extra_context):
        context = {}

        if request.GET:
            form = form_class(data=request.GET)
        else:
            form = form_class()

        if form.is_valid():
            context.setdefault('data', form.get_context())

        context.setdefault('form', form)
        context.update(extra_context)

        templates = self.template_path(template_name)
        return self.render(request, templates, context)


class RankingSite(Application):

    kwargs_form_class = RankingConfigurationForm

    def __init__(self, name='ranking', app_name='ranking', **kwargs):
        super(RankingSite, self).__init__(
            name=name, app_name=app_name, **kwargs)

    def get_urls(self):
        urlpatterns = [
            url(r'^$', rank.IndexView.as_view(), name='index'),
            url(r'^(?P<year>\d{4})/', include([
                url(r'^$', rank.YearView.as_view(), name='year'),
                url(r'^(?P<month>[a-z]{3})/', include([
                    url(r'^$', rank.MonthView.as_view(), name='month'),
                    url(r'^(?P<day>\d{1,2})/', include([
                        url(r'^$', rank.DayView.as_view(), name='day'),
                        url(r'^(?P<slug>[^/]+)/', include([
                            url(r'^$', rank.DivisionView.as_view(), name='rank'),
                            url(r'^(?P<team>[^/]+)/$', rank.TeamView.as_view(), name='team'),
                        ])),
                    ])),
                ])),
            ])),
        ]
        return urlpatterns


competition = CompetitionSite()
registration = RegistrationSite()
calculator = TournamentCalculatorSite()
ranking = RankingSite()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import django.db.models.deletion
import touchtechnology.common.db.models
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Club',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(max_length=255, verbose_name='Title')),
                ('short_title', models.CharField(help_text='This is used in navigation menus instead of the longer title value.', max_length=100, verbose_name='Short title', blank=True)),
                ('slug', models.SlugField(max_length=255, verbose_name='Slug')),
                ('slug_locked', models.BooleanField(default=False, verbose_name='Slug locked')),
                ('email', models.EmailField(max_length=255, blank=True)),
                ('website', models.URLField(max_length=255, blank=True)),
                ('twitter', models.CharField(help_text=b'Official Twitter name for use in social "mentions"', max_length=50, blank=True)),
                ('abbreviation', models.CharField(help_text=b'Optional 3-letter abbreviation to be used on scoreboards.', max_length=3, blank=True)),
            ],
            options={
                'ordering': ('title',),
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='ClubAssociation',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('club', touchtechnology.common.db.models.ForeignKey(related_name=b'staff', to='competition.Club')),
            ],
            options={
                'ordering': ('person__last_name', 'person__first_name'),
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='ClubRole',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=50)),
            ],
            options={
                'ordering': ('name',),
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Competition',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(max_length=255, verbose_name='Title')),
                ('short_title', models.CharField(help_text='This is used in navigation menus instead of the longer title value.', max_length=100, verbose_name='Short title', blank=True)),
                ('slug', models.SlugField(max_length=255, verbose_name='Slug')),
                ('slug_locked', models.BooleanField(default=False, verbose_name='Slug locked')),
                ('order', models.PositiveIntegerField(default=1)),
                ('copy', touchtechnology.common.db.models.HTMLField(blank=True)),
                ('enabled', models.BooleanField(default=True)),
                ('clubs', touchtechnology.common.db.models.ManyToManyField(related_name=b'competitions', null=True, to='competition.Club', blank=True)),
            ],
            options={
                'ordering': ('order',),
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Division',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(max_length=255, verbose_name='Title')),
                ('short_title', models.CharField(help_text='This is used in navigation menus instead of the longer title value.', max_length=100, verbose_name='Short title', blank=True)),
                ('slug', models.SlugField(max_length=255, verbose_name='Slug')),
                ('slug_locked', models.BooleanField(default=False, verbose_name='Slug locked')),
                ('order', models.PositiveIntegerField(default=1)),
                ('points_formula', models.TextField(null=True, verbose_name='Points system', blank=True)),
                ('bonus_points_formula', models.TextField(null=True, blank=True)),
                ('forfeit_for_score', models.SmallIntegerField(null=True)),
                ('forfeit_against_score', models.SmallIntegerField(null=True)),
                ('include_forfeits_in_played', models.BooleanField(default=False)),
                ('games_per_day', models.SmallIntegerField(help_text=b'\n        In Tournament mode, specify how many matches per day should be scheduled\n        by the automatic draw generator.', null=True)),
            ],
            options={
                'ordering': ('order',),
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='DivisionExclusionDate',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('date', touchtechnology.common.db.models.DateField()),
                ('division', touchtechnology.common.db.models.ForeignKey(related_name=b'exclusions', to='competition.Division')),
            ],
            options={
                'verbose_name': 'Exclusion Date',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='DrawFormat',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=50)),
                ('text', models.TextField(verbose_name='Formula')),
                ('teams', models.PositiveIntegerField(null=True, blank=True)),
                ('is_final', models.BooleanField(default=False)),
            ],
            options={
                'ordering': ('is_final', 'teams', 'name'),
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='LadderEntry',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('played', models.SmallIntegerField(default=0)),
                ('win', models.SmallIntegerField(default=0)),
                ('loss', models.SmallIntegerField(default=0)),
                ('draw', models.SmallIntegerField(default=0)),
                ('bye', models.SmallIntegerField(default=0)),
                ('forfeit_for', models.SmallIntegerField(default=0)),
                ('forfeit_against', models.SmallIntegerField(default=0)),
                ('score_for', models.SmallIntegerField(default=0)),
                ('score_against', models.SmallIntegerField(default=0)),
                ('bonus_points', models.SmallIntegerField(default=0)),
                ('points', models.DecimalField(default=0, max_digits=6, decimal_places=3)),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='LadderSummary',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('played', models.SmallIntegerField(default=0)),
                ('win', models.SmallIntegerField(default=0)),
                ('loss', models.SmallIntegerField(default=0)),
                ('draw', models.SmallIntegerField(default=0)),
                ('bye', models.SmallIntegerField(default=0)),
                ('forfeit_for', models.SmallIntegerField(default=0)),
                ('forfeit_against', models.SmallIntegerField(default=0)),
                ('score_for', models.SmallIntegerField(default=0)),
                ('score_against', models.SmallIntegerField(default=0)),
                ('bonus_points', models.SmallIntegerField(default=0)),
                ('points', models.DecimalField(default=0, max_digits=6, decimal_places=3)),
                ('difference', models.DecimalField(default=0, max_digits=6, decimal_places=3)),
                ('percentage', models.DecimalField(null=True, max_digits=10, decimal_places=2)),
            ],
            options={
                'ordering': ('stage', '-points', '-difference', '-percentage', 'team__title'),
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Match',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('label', models.CharField(max_length=100, null=True, blank=True)),
                ('home_team_eval', models.CharField(max_length=10, null=True, blank=True)),
                ('away_team_eval', models.CharField(max_length=10, null=True, blank=True)),
                ('evaluated', models.NullBooleanField()),
                ('is_washout', models.BooleanField(default=False)),
                ('date', touchtechnology.common.db.models.DateField(null=True, blank=True)),
                ('time', touchtechnology.common.db.models.TimeField(null=True, blank=True)),
                ('datetime', touchtechnology.common.db.models.DateTimeField(null=True, blank=True)),
                ('home_team_score', models.IntegerField(null=True, blank=True)),
                ('away_team_score', models.IntegerField(null=True, blank=True)),
                ('is_bye', models.BooleanField(default=False)),
                ('bye_processed', models.BooleanField(default=False)),
                ('is_forfeit', models.BooleanField(default=False)),
                ('round', models.IntegerField(null=True, blank=True)),
                ('include_in_ladder', models.BooleanField(default=True)),
            ],
            options={
                'ordering': ('date', 'stage', 'round', 'is_bye', 'time', 'play_at__ground__order', 'id'),
                'verbose_name_plural': 'matches',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Person',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('first_name', models.CharField(max_length=30)),
                ('last_name', models.CharField(max_length=30)),
                ('gender', models.CharField(max_length=1, choices=[(b'M', 'Male'), (b'F', 'Female')])),
                ('date_of_birth', touchtechnology.common.db.models.DateField(null=True, blank=True)),
                ('email', models.EmailField(max_length=75, blank=True)),
                ('home_phone', models.CharField(max_length=30, blank=True)),
                ('work_phone', models.CharField(max_length=30, blank=True)),
                ('mobile_phone', models.CharField(max_length=30, blank=True)),
                ('club', touchtechnology.common.db.models.ForeignKey(related_name=b'members', to='competition.Club')),
            ],
            options={
                'ordering': ('last_name', 'first_name'),
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Place',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(max_length=255, verbose_name='Title')),
                ('short_title', models.CharField(help_text='This is used in navigation menus instead of the longer title value.', max_length=100, verbose_name='Short title', blank=True)),
                ('slug', models.SlugField(max_length=255, verbose_name='Slug')),
                ('slug_locked', models.BooleanField(default=False, verbose_name='Slug locked')),
                ('order', models.PositiveIntegerField(default=1)),
                ('abbreviation', models.CharField(max_length=20, null=True, blank=True)),
                ('latlng', touchtechnology.common.db.models.LocationField(max_length=100)),
                ('timezone', models.CharField(blank=True, max_length=50, null=True)),
            ],
            options={
                'ordering': ('order',),
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Ground',
            fields=[
                ('place_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='competition.Place')),
            ],
            options={
                'ordering': ('order',),
                'abstract': False,
            },
            bases=('competition.place',),
        ),
        migrations.CreateModel(
            name='Season',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(max_length=255, verbose_name='Title')),
                ('short_title', models.CharField(help_text='This is used in navigation menus instead of the longer title value.', max_length=100, verbose_name='Short title', blank=True)),
                ('slug', models.SlugField(max_length=255, verbose_name='Slug')),
                ('slug_locked', models.BooleanField(default=False, verbose_name='Slug locked')),
                ('order', models.PositiveIntegerField(default=1)),
                ('hashtag', models.CharField(help_text=b'Your official tag for social media promotions, excluding the # symbol.', max_length=30, null=True, verbose_name=b'Hash Tag', blank=True)),
                ('enabled', models.BooleanField(default=True)),
                ('start_date', touchtechnology.common.db.models.DateField(null=True, blank=True)),
                ('mode', models.IntegerField(default=2, help_text=b'Used by the draw wizard to help you set your match dates & times automatically.', choices=[(2, 'Season'), (3, 'Tournament')])),
                ('statistics', models.BooleanField(default=True, help_text=b'Set to No if you do not wish to keep scoring or most valuable player statistics.')),
                ('mvp_results_public', touchtechnology.common.db.models.DateTimeField(help_text=b'Set to prevent the front-end site revealing MVP points before the specified time.', null=True, blank=True)),
                ('complete', models.BooleanField(default=False, help_text=b'Set to indicate this season is in the past. Optimises progression calculations.')),
                ('timezone', models.CharField(blank=True, max_length=50, null=True)),
                ('competition', touchtechnology.common.db.models.ForeignKey(related_name=b'seasons', to='competition.Competition')),
            ],
            options={
                'ordering': ('order',),
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='SeasonAssociation',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('club', models.ForeignKey(related_name=b'officials', to='competition.Club')),
                ('person', touchtechnology.common.db.models.ForeignKey(to='competition.Person')),
                ('roles', touchtechnology.common.db.models.ManyToManyField(to='competition.ClubRole')),
                ('season', models.ForeignKey(related_name=b'officials', to='competition.Season')),
            ],
            options={
                'ordering': ('club', 'season', 'person__last_name', 'person__first_name'),
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='SeasonExclusionDate',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('date', touchtechnology.common.db.models.DateField()),
                ('season', touchtechnology.common.db.models.ForeignKey(related_name=b'exclusions', to='competition.Season')),
            ],
            options={
                'verbose_name': 'Exclusion Date',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='SeasonMatchTime',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('start', touchtechnology.common.db.models.TimeField()),
                ('interval', models.IntegerField()),
                ('count', models.IntegerField()),
                ('start_date', touchtechnology.common.db.models.DateField(null=True, verbose_name=b'From', blank=True)),
                ('end_date', touchtechnology.common.db.models.DateField(null=True, verbose_name=b'Until', blank=True)),
                ('season', touchtechnology.common.db.models.ForeignKey(related_name=b'timeslots', to='competition.Season')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='SimpleScoreMatchStatistic',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('number', models.IntegerField(null=True, blank=True)),
                ('played', models.SmallIntegerField(default=0)),
                ('points', models.SmallIntegerField(default=0)),
                ('mvp', models.SmallIntegerField(default=0)),
                ('match', touchtechnology.common.db.models.ForeignKey(related_name=b'statistics', to='competition.Match')),
                ('player', touchtechnology.common.db.models.ForeignKey(related_name=b'statistics', to='competition.Person')),
            ],
            options={
                'ordering': ('match', 'number'),
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Stage',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(max_length=255, verbose_name='Title')),
                ('short_title', models.CharField(help_text='This is used in navigation menus instead of the longer title value.', max_length=100, verbose_name='Short title', blank=True)),
                ('slug', models.SlugField(max_length=255, verbose_name='Slug')),
                ('slug_locked', models.BooleanField(default=False, verbose_name='Slug locked')),
                ('order', models.PositiveIntegerField(default=1)),
                ('keep_ladder', models.BooleanField(default=True, help_text=b'\n        Set this to <b>No</b> if this stage does not need to keep a competition ladder.<br />\n        Usually set to No for a Final Series or a Knockout stage.\n        ', verbose_name=b'Keep a ladder')),
                ('scale_group_points', models.BooleanField(default=False, help_text=b'\n        In stages with multiple pools, adjust points in the smaller groups to\n        compensate for the reduced opportunity to score points.<br />\n        You <strong>should</strong> also set 0 points for Bye matches.')),
                ('carry_ladder', models.BooleanField(default=False, help_text=b'\n        Set this to <b>Yes</b> if this stage should carry over values from the previous stage.\n        ', verbose_name=b'Carry over points')),
                ('keep_mvp', models.BooleanField(default=True, help_text=b'\n        Set this to <b>No</b> if this stage does not need to keep track of MVP points.<br />\n        Usually set to No for a Final Series.\n        ', verbose_name=b'Keep MVP stats')),
                ('division', touchtechnology.common.db.models.ForeignKey(related_name=b'stages', to='competition.Division')),
                ('follows', touchtechnology.common.db.models.ForeignKey(related_name=b'preceeds', on_delete=django.db.models.deletion.SET_NULL, blank=True, to='competition.Stage', help_text=b'\n        When progressing teams into this stage, which earlier stage should be used for\n        determining positions.<br />\n        Default is the immediately preceeding stage.\n        ', null=True)),
                ('matches_needing_printing', touchtechnology.common.db.models.ManyToManyField(related_name=b'to_be_printed', to='competition.Match', blank=True)),
            ],
            options={
                'ordering': ('order',),
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='StageGroup',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(max_length=255, verbose_name='Title')),
                ('short_title', models.CharField(help_text='This is used in navigation menus instead of the longer title value.', max_length=100, verbose_name='Short title', blank=True)),
                ('slug', models.SlugField(max_length=255, verbose_name='Slug')),
                ('slug_locked', models.BooleanField(default=False, verbose_name='Slug locked')),
                ('order', models.PositiveIntegerField(default=1)),
                ('carry_ladder', models.BooleanField(default=False, help_text=b'\n        Set this to <b>Yes</b> if the ladder for this pool should carry over values from\n        the previous stage.<br />\n        Will only apply for matches played against teams that are now in this group.\n        ', verbose_name=b'Carry over points')),
                ('stage', touchtechnology.common.db.models.ForeignKey(related_name=b'pools', to='competition.Stage')),
            ],
            options={
                'ordering': ('order',),
                'verbose_name': 'Pool',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Team',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(max_length=255, verbose_name='Title')),
                ('short_title', models.CharField(help_text='This is used in navigation menus instead of the longer title value.', max_length=100, verbose_name='Short title', blank=True)),
                ('slug', models.SlugField(max_length=255, verbose_name='Slug')),
                ('slug_locked', models.BooleanField(default=False, verbose_name='Slug locked')),
                ('order', models.PositiveIntegerField(default=1)),
                ('names_locked', models.BooleanField(default=False, help_text=b'\n        When the team name is locked, the team manager will not be able to change their team name.<br />\n        As a tournament manager you can always change the names.')),
                ('timeslots_after', touchtechnology.common.db.models.TimeField(help_text=b'Specify the earliest time that this team can play. Leave blank for no preference.', null=True, verbose_name=b'Start after', blank=True)),
                ('timeslots_before', touchtechnology.common.db.models.TimeField(help_text=b'Specify the latest time that this team can play. Leave blank for no preference.', null=True, verbose_name=b'Start before', blank=True)),
                ('club', touchtechnology.common.db.models.ForeignKey(related_name=b'teams', on_delete=django.db.models.deletion.SET_NULL, blank=True, to='competition.Club', null=True)),
                ('division', touchtechnology.common.db.models.ForeignKey(related_name=b'teams', blank=True, to='competition.Division', null=True)),
                ('stage_group', touchtechnology.common.db.models.ForeignKey(related_name=b'teams', on_delete=django.db.models.deletion.SET_NULL, verbose_name=b'Pool', blank=True, to='competition.StageGroup', null=True)),
                ('team_clashes', touchtechnology.common.db.models.ManyToManyField(help_text=b'Select any teams that must not play at the same time.', related_name='team_clashes_rel_+', verbose_name=b"Don't clash", to='competition.Team', blank=True)),
            ],
            options={
                'ordering': ('division__order', 'stage_group__order', 'order'),
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='TeamAssociation',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('number', models.IntegerField(null=True, blank=True)),
                ('is_player', models.BooleanField(default=True)),
                ('person', touchtechnology.common.db.models.ForeignKey(to='competition.Person', null=True)),
            ],
            options={
                'ordering': ('-is_player', 'number', 'person__last_name', 'person__first_name'),
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='TeamRole',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=50)),
                ('competition', touchtechnology.common.db.models.ForeignKey(related_name=b'team_roles', to='competition.Competition')),
            ],
            options={
                'ordering': ('name',),
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='UndecidedTeam',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('formula', models.CharField(max_length=20, blank=True)),
                ('label', models.CharField(max_length=30, blank=True)),
                ('stage', touchtechnology.common.db.models.ForeignKey(related_name=b'undecided_teams', to='competition.Stage')),
                ('stage_group', touchtechnology.common.db.models.ForeignKey(related_name=b'undecided_teams', on_delete=django.db.models.deletion.SET_NULL, verbose_name=b'Pool', blank=True, to='competition.StageGroup', null=True)),
            ],
            options={
                'ordering': ('stage_group', 'formula'),
                'verbose_name': 'team',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Venue',
            fields=[
                ('place_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='competition.Place')),
                ('season', touchtechnology.common.db.models.ForeignKey(related_name=b'venues', to='competition.Season')),
            ],
            options={
                'ordering': ('order',),
                'abstract': False,
            },
            bases=('competition.place',),
        ),
        migrations.AddField(
            model_name='teamassociation',
            name='roles',
            field=touchtechnology.common.db.models.ManyToManyField(to='competition.TeamRole', blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='teamassociation',
            name='team',
            field=touchtechnology.common.db.models.ForeignKey(related_name=b'people', to='competition.Team'),
            preserve_default=True,
        ),
        migrations.AlterUniqueTogether(
            name='teamassociation',
            unique_together=set([('team', 'person')]),
        ),
        migrations.AlterUniqueTogether(
            name='team',
            unique_together=set([('title', 'division')]),
        ),
        migrations.AlterUniqueTogether(
            name='stagegroup',
            unique_together=set([('stage', 'order')]),
        ),
        migrations.AlterUniqueTogether(
            name='stage',
            unique_together=set([('division', 'slug')]),
        ),
        migrations.AlterUniqueTogether(
            name='seasonexclusiondate',
            unique_together=set([('season', 'date')]),
        ),
        migrations.AlterUniqueTogether(
            name='seasonassociation',
            unique_together=set([('season', 'person')]),
        ),
        migrations.AddField(
            model_name='match',
            name='away_team',
            field=touchtechnology.common.db.models.ForeignKey(related_name=b'away_games', blank=True, to='competition.Team', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='match',
            name='away_team_eval_related',
            field=touchtechnology.common.db.models.ForeignKey(related_name=b'+away_team_eval', blank=True, to='competition.Match', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='match',
            name='away_team_undecided',
            field=touchtechnology.common.db.models.ForeignKey(related_name=b'away_games', blank=True, to='competition.UndecidedTeam', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='match',
            name='forfeit_winner',
            field=touchtechnology.common.db.models.ForeignKey(related_name=b'+team', blank=True, to='competition.Team', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='match',
            name='home_team',
            field=touchtechnology.common.db.models.ForeignKey(related_name=b'home_games', blank=True, to='competition.Team', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='match',
            name='home_team_eval_related',
            field=touchtechnology.common.db.models.ForeignKey(related_name=b'+home_team_eval', blank=True, to='competition.Match', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='match',
            name='home_team_undecided',
            field=touchtechnology.common.db.models.ForeignKey(related_name=b'home_games', blank=True, to='competition.UndecidedTeam', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='match',
            name='play_at',
            field=touchtechnology.common.db.models.ForeignKey(on_delete=django.db.models.deletion.SET_NULL, to='competition.Place', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='match',
            name='stage',
            field=touchtechnology.common.db.models.ForeignKey(related_name=b'matches', to='competition.Stage', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='match',
            name='stage_group',
            field=touchtechnology.common.db.models.ForeignKey(related_name=b'matches', verbose_name=b'Pool', blank=True, to='competition.StageGroup', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='laddersummary',
            name='stage',
            field=touchtechnology.common.db.models.ForeignKey(related_name=b'ladder_summary', to='competition.Stage'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='laddersummary',
            name='stage_group',
            field=touchtechnology.common.db.models.ForeignKey(related_name=b'ladder_summary', to='competition.StageGroup', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='laddersummary',
            name='team',
            field=touchtechnology.common.db.models.ForeignKey(related_name=b'ladder_summary', to='competition.Team'),
            preserve_default=True,
        ),
        migrations.AlterUniqueTogether(
            name='laddersummary',
            unique_together=set([('stage', 'team')]),
        ),
        migrations.AddField(
            model_name='ladderentry',
            name='match',
            field=touchtechnology.common.db.models.ForeignKey(related_name=b'ladder_entries', to='competition.Match'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='ladderentry',
            name='opponent',
            field=touchtechnology.common.db.models.ForeignKey(to='competition.Team', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='ladderentry',
            name='team',
            field=touchtechnology.common.db.models.ForeignKey(related_name=b'ladder_entries', to='competition.Team'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='ground',
            name='venue',
            field=touchtechnology.common.db.models.ForeignKey(related_name=b'grounds', to='competition.Venue'),
            preserve_default=True,
        ),
        migrations.AlterUniqueTogether(
            name='divisionexclusiondate',
            unique_together=set([('division', 'date')]),
        ),
        migrations.AddField(
            model_name='division',
            name='season',
            field=touchtechnology.common.db.models.ForeignKey(related_name=b'divisions', to='competition.Season'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='clubrole',
            name='competition',
            field=touchtechnology.common.db.models.ForeignKey(related_name=b'club_roles', to='competition.Competition'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='clubassociation',
            name='person',
            field=touchtechnology.common.db.models.ForeignKey(to='competition.Person'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='clubassociation',
            name='roles',
            field=touchtechnology.common.db.models.ManyToManyField(to='competition.ClubRole'),
            preserve_default=True,
        ),
        migrations.AlterUniqueTogether(
            name='clubassociation',
            unique_together=set([('club', 'person')]),
        ),
        migrations.AddField(
            model_name='club',
            name='primary',
            field=touchtechnology.common.db.models.ForeignKey(related_name=b'+club', blank=True, to='competition.Person', help_text=b'Appears on the front-end with other club information.', null=True, verbose_name=b'Primary contact'),
            preserve_default=True,
        ),
        migrations.CreateModel(
            name='ByeTeam',
            fields=[
            ],
            options={
                'proxy': True,
            },
            bases=('competition.team',),
        ),
    ]



import logging

from django.core.cache.backends.base import DEFAULT_TIMEOUT
from redis.exceptions import ConnectionError
from redis_cache.cache import RedisCache as BaseRedisCache

logger = logging.getLogger(__name__)


class RedisCache(BaseRedisCache):
    """
    The standard redis_cache.RedisCache will throw ConnectionError exceptions
    when the backend cache is unavailable to it. This implementation will catch
    the exceptions and log the failure.

    We log at ERROR level all attempts that fail due to ConnectionError.
    We log at DEBUG level all attempts that succeed.
    """

    def get(self, key, default=None, version=None):
        """
        Wrap the underlying ``get``, returning ``default`` when the backend is
        unavailable.
        """
        try:
            hit = super(RedisCache, self).get(
                key, default=default, version=version)
        except ConnectionError as exc:
            hit = default
            logger.error('redis="get", key="%s", default="%s", version="%s", '
                         'result="miss", exception="%s"',
                         key, default, version, exc)
        else:
            logger.debug('redis="get", key="%s", default="%s", version="%s", '
                         'result="hit"', key, default, version)
        return hit

    def set(self, key, value, timeout=DEFAULT_TIMEOUT, version=None):
        """
        Wrap the underlying ``set``, silently passing when the backend is
        unavailable.
        """
        if timeout == DEFAULT_TIMEOUT:
            timeout = self.default_timeout

        try:
            super(RedisCache, self).set(
                key, value, timeout=timeout, version=version)
        except ConnectionError as exc:
            logger.error('redis="set", key="%s", version="%s", timeout="%s", '
                         'result="miss", exception="%s"',
                         key, version, timeout, exc)
        else:
            logger.debug('redis="set", key="%s", version="%s", timeout="%s", '
                         'result="hit"', key, version, timeout)

    def delete(self, key, version=None):
        """
        Wrap the underlying ``delete``, silently passing when the backend is
        unavailable.
        """
        try:
            super(RedisCache, self).delete(key, version=version)
        except ConnectionError as exc:
            logger.error('redis="delete", key="%s", version="%s", '
                         'result="miss", exception="%s"',
                         key, version, exc)

    def clear(self):
        """
        Wrap the underlying ``clear``, silently passing when the backend is
        unavailable.
        """
        try:
            super(RedisCache, self).clear()
        except ConnectionError as exc:
            logger.error('redis="clear", result="miss", exception="%s"', exc)



from __future__ import unicode_literals

import unittest
from datetime import date, datetime, time

import django
import pytz
import six
from django.conf import settings
from django.template import Context, Template
from django.urls import reverse
from test_plus import TestCase as BaseTestCase
from touchtechnology.common.tests.factories import UserFactory
from tournamentcontrol.competition.models import Team
from tournamentcontrol.competition.tests import factories
from tournamentcontrol.competition.utils import round_robin


class TestCase(BaseTestCase):

    def setUp(self):
        super(TestCase, self).setUp()
        self.superuser = UserFactory.create(is_staff=True, is_superuser=True)

    def assertGoodEditView(self, viewname, *args, **kwargs):
        test_query_count = kwargs.pop('test_query_count', 50)
        self.assertLoginRequired(viewname, *args)
        with self.login(self.superuser):
            self.assertGoodView(
                viewname, test_query_count=test_query_count, *args)
            if kwargs:
                self.post(viewname, *args, **kwargs)
                self.response_302()

    def assertGoodDeleteView(self, viewname, *args):
        self.assertLoginRequired(viewname, *args)
        with self.login(self.superuser):
            self.get(viewname, *args)
            self.response_405()
            self.post(viewname, *args)
            self.response_302()

    def assertGoodNamespace(self, instance, **kwargs):
        namespace = instance._get_admin_namespace()
        args = instance._get_url_args()
        self.assertGoodEditView('%s:add' % namespace, *args[:-1], **kwargs)
        self.assertGoodEditView('%s:edit' % namespace, *args, **kwargs)
        self.assertGoodDeleteView('%s:delete' % namespace, *args)


class TemplateTests(TestCase):

    @classmethod
    def setUpClass(cls):
        super(TemplateTests, cls).setUpClass()
        stage1 = factories.StageFactory.create()
        stage2 = factories.StageFactory.create(division=stage1.division, follows=stage1)
        ground = factories.GroundFactory.create(venue__season=stage2.division.season)
        cls.team = factories.TeamFactory.create(
            title="Chees\u00e9\u00a0&\u00a0Crackers", division=stage1.division)
        factories.MatchFactory.create(
            home_team=None, away_team=None, home_team_eval="P1", away_team_eval="P2",
            stage=stage2, date=date(2017, 2, 13), time=time(9, 0), play_at=ground,
            datetime=datetime(2017, 2, 13, 9, tzinfo=pytz.utc))
        factories.MatchFactory.create(
            home_team=cls.team, away_team=None, home_team_eval="P3", away_team_eval="P4",
            stage=stage2, date=date(2017, 2, 13), time=time(10, 0), play_at=ground,
            datetime=datetime(2017, 2, 13, 10, tzinfo=pytz.utc))
        cls.stage = stage2
        cls.division = stage2.division
        cls.season = cls.division.season
        cls.competition = cls.season.competition

    @classmethod
    def tearDownClass(cls):
        super(TemplateTests, cls).tearDownClass()

    def test_team_title(self):
        template = Template('{% load common %}{{ team.title|htmlentities }}')
        context = Context({'team': self.team})
        output = template.render(context)
        self.assertEqual(output, 'Chees&eacute;&nbsp;&amp;&nbsp;Crackers')

    def test_match_runsheet(self):
        self.assertLoginRequired(
            'admin:fixja:match-runsheet', self.competition.pk, self.season.pk, '20170213')
        with self.login(self.superuser):
            self.assertGoodView(
                'admin:fixja:match-runsheet', self.competition.pk, self.season.pk, '20170213')
            self.assertResponseContains('<td class="team">1st</td>')
            self.assertResponseContains('<td class="team">2nd </td>')  # whitespace deliberate
            self.assertResponseContains(
                '<td class="team">Chees&eacute;&nbsp;&amp;&nbsp;Crackers</td>')

    def test_match_grid(self):
        self.assertLoginRequired(
            'admin:fixja:match-grid', self.competition.pk, self.season.pk, '20170213', 'html')
        with self.login(self.superuser):
            self.assertGoodView(
                'admin:fixja:match-grid', self.competition.pk, self.season.pk, '20170213', 'html')
            self.assertResponseContains(
                '<h4>%s<br /><small>%s</small></h4>' % (self.division, self.stage))
            self.assertResponseContains('<p>1st <br>2nd</p>')  # whitespace & br style deliberate
            self.assertResponseContains('<p>Chees&eacute;&nbsp;&amp;&nbsp;Crackers<br>4th</p>')


class GoodViewTests(TestCase):

    def test_reorder_down(self):
        stage = factories.StageFactory.create()
        pool_a, pool_b, pool_c = \
            factories.StageGroupFactory.create_batch(stage=stage, size=3)

        with self.login(self.superuser):
            # Without a HTTP_REFERER set this should throw a 404
            self.get('admin:fixja:reorder', 'stagegroup', pool_b.pk, 'down')
            self.response_404()

            # The first time we try to shift this down it will work because it
            # isn't last in it's set.
            self.get('admin:fixja:reorder', 'stagegroup', pool_b.pk, 'down',
                     extra=dict(HTTP_REFERER='http://testserver/'))
            self.response_302()

            # This time, however, it should fail because we're now last.
            self.get('admin:fixja:reorder', 'stagegroup', pool_b.pk, 'down',
                     extra=dict(HTTP_REFERER='http://testserver/'))
            self.response_404()

    def test_reorder_up(self):
        stage = factories.StageFactory.create()
        pool_a, pool_b, pool_c = \
            factories.StageGroupFactory.create_batch(stage=stage, size=3)

        with self.login(self.superuser):
            # Without a HTTP_REFERER set this should throw a 404
            self.get('admin:fixja:reorder', 'stagegroup', pool_b.pk, 'up')
            self.response_404()

            # The first time we try to shift this up it will work because it
            # isn't first in it's set.
            self.get('admin:fixja:reorder', 'stagegroup', pool_b.pk, 'up',
                     extra=dict(HTTP_REFERER='http://testserver/'))
            self.response_302()

            # This time, however, it should fail because we're now first.
            self.get('admin:fixja:reorder', 'stagegroup', pool_b.pk, 'up',
                     extra=dict(HTTP_REFERER='http://testserver/'))
            self.response_404()

    def test_reorder_left(self):
        stage = factories.StageFactory.create()
        pool_a, pool_b, pool_c = \
            factories.StageGroupFactory.create_batch(stage=stage, size=3)

        with self.login(self.superuser):
            # A direction of "left" is invalid and should throw a 404
            self.get('admin:fixja:reorder', pool_b.pk, 'left',
                     extra=dict(HTTP_REFERER='http://testserver/'))
            self.response_404()

    def test_index(self):
        self.assertLoginRequired('admin:fixja:index')
        with self.login(self.superuser):
            self.get('admin:fixja:index')
            self.response_302()

    def test_competition(self):
        competition = factories.CompetitionFactory.create()
        self.assertLoginRequired('admin:fixja:competition:list')
        with self.login(self.superuser):
            self.assertGoodView('admin:fixja:competition:list')
        self.assertGoodNamespace(competition)

    def test_clubrole(self):
        role = factories.ClubRoleFactory.create()
        self.assertGoodNamespace(role, data={'name': 'Club Role'})

    def test_teamrole(self):
        role = factories.TeamRoleFactory.create()
        self.assertGoodNamespace(role, data={'name': 'Team Role'})

    def test_season(self):
        season = factories.SeasonFactory.create()
        self.assertGoodNamespace(season)

    def test_season_exclusion(self):
        exclusion = factories.SeasonExclusionDateFactory.create()
        self.assertGoodNamespace(exclusion)

    def test_timeslot(self):
        matchtime = factories.SeasonMatchTimeFactory.create()
        self.assertGoodNamespace(matchtime)

    def test_venue(self):
        venue = factories.VenueFactory.create()
        self.assertGoodNamespace(venue)

    def test_ground(self):
        ground = factories.GroundFactory.create()
        self.assertGoodNamespace(ground)

    def test_division(self):
        division = factories.DivisionFactory.create()
        self.assertGoodNamespace(division)

    def test_division_exclusion(self):
        exclusion = factories.DivisionExclusionDateFactory.create()
        self.assertGoodNamespace(exclusion)

    def test_team(self):
        team = factories.TeamFactory.create()
        self.assertGoodNamespace(team)

    def test_teamassociation(self):
        team_association = factories.TeamAssociationFactory.create()
        self.assertGoodNamespace(team_association)

    def test_stage(self):
        stage = factories.StageFactory.create()
        self.assertGoodNamespace(stage, test_query_count=100)

    def test_match(self):
        match = factories.MatchFactory.create()
        self.assertGoodNamespace(match)

    def test_pool(self):
        pool = factories.StageGroupFactory.create()
        self.assertGoodNamespace(pool)

    @unittest.skip("django-guardian bug - see "
                   "https://github.com/django-guardian/django-guardian/issues/519")
    def test_club(self):
        club = factories.ClubFactory.create()
        self.assertLoginRequired('admin:fixja:club:list')
        with self.login(self.superuser):
            self.assertGoodView('admin:fixja:club:list')
        self.assertGoodNamespace(club)

    def test_person(self):
        person = factories.PersonFactory.create()
        self.assertFalse(person.statistics.count())
        self.assertGoodNamespace(person)

    def test_format(self):
        draw_format = factories.DrawFormatFactory.create()
        self.assertLoginRequired('admin:fixja:format:list')
        with self.login(self.superuser):
            self.assertGoodView('admin:fixja:format:list')
        self.assertGoodNamespace(draw_format)

    @unittest.skip("Not refactored as a namespace yet")
    def test_list_drawformat(self):
        self.assertLoginRequired('admin:fixja:report:list')
        with self.login(self.superuser):
            self.assertGoodView('admin:fixja:report:list')

    # code coverage

    def test_season_report(self):
        season = factories.SeasonFactory.create()
        self.assertLoginRequired(
            'admin:fixja:season-report', season.competition_id, season.pk)
        with self.login(self.superuser):
            self.assertGoodView(
                'admin:fixja:season-report', season.competition_id, season.pk)

    def test_season_summary(self):
        season = factories.SeasonFactory.create()
        self.assertLoginRequired(
            'admin:fixja:season-summary', season.competition_id, season.pk)
        with self.login(self.superuser):
            self.assertGoodView(
                'admin:fixja:season-summary', season.competition_id, season.pk)

    def test_match_schedule_season(self):
        season = factories.SeasonFactory.create()
        self.assertLoginRequired(
            'admin:fixja:match-schedule', season.competition_id, season.pk, '20170213')
        with self.login(self.superuser):
            self.assertGoodView(
                'admin:fixja:match-schedule', season.competition_id, season.pk, '20170213')

    def test_match_schedule_division(self):
        division = factories.DivisionFactory.create()
        self.assertLoginRequired(
            'admin:fixja:match-schedule',
            division.season.competition_id, division.season_id, '20170213', division.pk)
        with self.login(self.superuser):
            self.assertGoodView(
                'admin:fixja:match-schedule',
                division.season.competition_id, division.season_id, '20170213', division.pk)

    def test_progress_teams(self):
        division = factories.DivisionFactory.create()
        teams = factories.TeamFactory.create_batch(division=division, size=6)

        round_games, finals = factories.StageFactory.create_batch(
            division=division, size=2)

        for round in round_robin(teams):
            for home_team, away_team in round:
                factories.MatchFactory.create(
                    stage=round_games,
                    home_team=home_team,
                    away_team=away_team,
                )

        # Easier to just build the 3 matches required for progression than to
        # automate it more extensively.
        semi_1 = factories.MatchFactory.create(
            stage=finals, label='Semi 1', round=6,
            home_team=None, home_team_eval='P1',
            away_team=None, away_team_eval='P4')
        semi_2 = factories.MatchFactory.create(
            stage=finals, label='Semi 2', round=6,
            home_team=None, home_team_eval='P2',
            away_team=None, away_team_eval='P3')
        factories.MatchFactory.create(
            stage=finals, label='Final', round=7,
            home_team=None, home_team_eval='W', home_team_eval_related=semi_1,
            away_team=None, away_team_eval='W', away_team_eval_related=semi_2)

        self.assertLoginRequired(
            round_games._get_admin_namespace() + ':draw:progress',
            *round_games._get_url_args())

        with self.login(self.superuser):
            # The round games cannot be progressed, it's the first stage and
            # therefore can't have any undecided teams.
            self.get(round_games._get_admin_namespace() + ':draw:progress',
                     *round_games._get_url_args())
            self.response_410()

            # FIXME
            #
            # # The final series can't be progressed because there are results
            # # that need to be entered for matches in the preceding stage.
            # self.get(finals._get_admin_namespace() + ':draw:progress',
            #          *finals._get_url_args())
            # self.response_410()
            #
            # # Set random results for every unplayed match in the previous stage
            # # so that we can determine progressions.
            # for m in round_games.matches.all():
            #     m.home_team_score = random.randint(0, 20)
            #     m.away_team_score = random.randint(0, 20)

            # The final series progression should now be accessible.
            self.get(finals._get_admin_namespace() + ':draw:progress',
                     *finals._get_url_args())
            self.response_200()

    def test_edit_person(self):
        person = factories.PersonFactory.create()
        self.assertLoginRequired(
            'admin:fixja:club:person:add',
            person.club.pk)
        self.assertLoginRequired(
            'admin:fixja:club:person:edit',
            person.club.pk,
            person.pk)
        with self.login(self.superuser):
            self.assertGoodView(
                'admin:fixja:club:person:add',
                person.club.pk)
            self.assertGoodView(
                'admin:fixja:club:person:edit',
                person.club.pk,
                person.pk)

    def test_merge_person(self):
        "Create two people, destructively merge second into the first."
        club = factories.ClubFactory.create()

        # Not using the PersonFactory because it requires a User, we want a
        # bare Person instance without that relationship.
        member1 = club.members.create(first_name='Alice', last_name='User')

        # Our second Person should have the User property, because we want to
        # transfer that to the first.
        member2 = factories.PersonFactory.create(club=club)

        # Check our initial setup is as we expect.
        six.assertCountEqual(
            self,
            [
                ('Alice', 'User', None),
                (member2.first_name, member2.last_name, member2.user.pk),
            ],
            club.members.values_list('first_name', 'last_name', 'user'))

        # Exercise the merge view.
        with self.login(self.superuser):
            data = {
                'first_name': 'Alice',
                'last_name': 'User',
                'gender': 'F',
                'date_of_birth': '1983-04-27',
                'email': '',
                'home_phone': '',
                'work_phone': '',
                'mobile_phone': '',
                'user': '',
                'merge': member2.pk,
                'keep_old': 'false',
            }
            self.post('admin:fixja:club:person:merge', club.pk, member1.pk, data=data)
            self.response_302()

        # Check the database state after the merge.
        six.assertCountEqual(
            self,
            [
                ('Alice', 'User', member2.user.pk, 'F', date(1983, 4, 27)),
            ],
            club.members.values_list(
                'first_name', 'last_name', 'user', 'gender', 'date_of_birth'))

    def test_edit_clubassociation(self):
        association = factories.ClubAssociationFactory.create()
        self.assertLoginRequired(
            'admin:fixja:club:clubassociation:add',
            association.club.pk)
        self.assertLoginRequired(
            'admin:fixja:club:clubassociation:edit',
            association.club.pk,
            association.pk)
        with self.login(self.superuser):
            self.assertGoodView(
                'admin:fixja:club:clubassociation:add',
                association.club.pk)
            self.assertGoodView(
                'admin:fixja:club:clubassociation:edit',
                association.club.pk,
                association.pk)

    # def test_delete_competition(self):
    #     self.assertLoginRequired('admin:fixja:competition:delete', 1)
    #     with self.login(username='gary@touch.asn.au'):
    #         self.get('admin:fixja:competition:delete', 1)
    #         self.response_405()
    #         self.post('admin:fixja:competition:delete', 1)
    #         self.response_302()

    def test_perms_views(self):
        team = factories.TeamFactory.create()
        self.assertLoginRequired(
            team.division.season.competition._get_admin_namespace() + ':perms',
            *team._get_url_args()[:1])
        self.assertLoginRequired(
            team.division.season._get_admin_namespace() + ':perms',
            *team._get_url_args()[:2])
        # self.assertLoginRequired(
        #     team.division._get_admin_namespace() + ':perms',
        #     *team._get_url_args()[:3])
        self.assertLoginRequired(
            team._get_admin_namespace() + ':perms',
            *team._get_url_args())

        with self.login(self.superuser):
            self.assertGoodView(
                team.division.season.competition._get_admin_namespace() + ':perms',
                *team._get_url_args()[:1])
            self.assertGoodView(
                team.division.season._get_admin_namespace() + ':perms',
                *team._get_url_args()[:2])
            # self.assertGoodView(
            #     team.division._get_admin_namespace() + ':perms',
            #     *team._get_url_args()[:3])
            with self.assertNumQueriesLessThan(100):
                self.get(
                    team._get_admin_namespace() + ':perms',
                    *team._get_url_args())
            self.response_200()

    def test_registration_form(self):
        club = factories.ClubFactory.create()
        team = factories.TeamFactory.create(club=club)
        team.division.season.competition.clubs.add(club)
        self.assertLoginRequired('admin:fixja:club:registration-form',
                                 club.pk, team.division.season.pk, 'html')
        with self.login(self.superuser):
            self.assertGoodView('admin:fixja:club:registration-form',
                                club.pk, team.division.season.pk, 'html')

    @unittest.expectedFailure
    def test_edit_team_members(self):
        club = factories.ClubFactory.create()
        team = factories.TeamFactory.create(club=club)
        self.assertLoginRequired('admin:fixja:club:team:edit',
                                 club.pk, team.division.season.pk, team.pk)
        with self.login(self.superuser):
            self.assertGoodView('admin:fixja:club:team:edit',
                                club.pk, team.division.season.pk, team.pk)


class BackendTests(TestCase):

    def setUp(self):
        super(BackendTests, self).setUp()
        self.login(self.superuser)

    @unittest.skipIf(
        django.VERSION > (1, 8) and
        settings.DATABASES['default']['ENGINE'].endswith('psycopg2'),
        "Django 1.9+ & PostgreSQL errors with 'GROUP BY' on annotate")
    def test_task_0058_highest_point_scorer(self):
        division = factories.DivisionFactory.create()
        self.assertGoodView(
            division._get_admin_namespace() + ':scorers',
            *division._get_url_args())

    def test_add_division(self):
        """
        Using the admin interface, add a new division and check that it appears
        in the front-end. Later update the form data and repost to ensure the
        front-end is also updated.
        """
        season = factories.SeasonFactory.create()
        data = {
            'title': 'Mixed 4',
            'short_title': 'X4',
            'points_formula_0': '3',
            'points_formula_1': '2',
            'points_formula_2': '1',
            'points_formula_3': '',
            'points_formula_4': '',
            'points_formula_5': '',
            'forfeit_for_score': '5',
            'forfeit_against_score': '0',
            'include_forfeits_in_played': '1',
            'slug': '',
            'slug_locked': '0',
        }
        self.post(season._get_admin_namespace() + ':division:add',
                  *season._get_url_args(), data=data)
        self.response_302()

        with self.settings(
                ROOT_URLCONF='tournamentcontrol.competition.tests.urls'):
            self.assertGoodView('competition:season',
                                season.competition.slug, season.slug)
            self.assertResponseContains('Mixed 4', html=False)
            self.assertGoodView('competition:division',
                                season.competition.slug,
                                season.slug, 'mixed-4')
            self.assertResponseContains('Mixed 4', html=False)

        data.update({
            'title': '4th Division Mixed',
        })
        division = season.divisions.latest('pk')

        self.post(division._get_admin_namespace() + ':edit',
                  *division._get_url_args(), data=data)
        self.response_302()

        with self.settings(
                ROOT_URLCONF='tournamentcontrol.competition.tests.urls'):
            self.assertGoodView('competition:season',
                                season.competition.slug, season.slug)
            self.assertResponseNotContains('Mixed 4', html=False)
            self.assertResponseContains('4th Division Mixed', html=False)

    def test_update_club(self):
        """
        Using the admin interface, update an existing Club and ensure that the
        revised attribute is correctly updated.
        """
        club = factories.ClubFactory.create()
        data = {
            'title': 'Bare Back Riders',
            'short_title': '',
            'abbreviation': 'BBR',
            'website': '',
            'twitter': '',
            'primary': '',
            'slug': 'bare-back-riders',
            'slug_locked': '0',
        }
        self.post('admin:fixja:club:edit', club.pk, data=data)
        self.response_302()

        self.get('admin:fixja:club:list')
        self.assertResponseContains('Bare Back Riders', html=False)
        self.assertResponseContains('BBR', html=False)

    def test_match_detail(self):
        play_at = factories.GroundFactory.create()
        m = factories.MatchFactory.create(play_at=play_at, home_team_score=3, away_team_score=0)

        # Make sure home team has some players associated with it.
        factories.TeamAssociationFactory.create_batch(
            4, is_player=True, team=m.home_team, person__club=m.home_team.club)

        kwargs = {
            'match_id': m.pk,
            'stage_id': m.stage_id,
            'division_id': m.stage.division_id,
            'season_id': m.stage.division.season_id,
            'competition_id': m.stage.division.season.competition_id,
        }
        url = reverse(
            'admin:competition:competition:season:division:stage:match:detail',
            kwargs=kwargs)

        res = self.client.get(url)

        # After GET ensure away_team has no players
        text = '''
        <td colspan="5" class="no_results">
            There are no registered players in this team.
        </td>
        '''
        self.assertContains(res, text, html=True)

        data = {
            'home-0-number': '3',
            'home-0-id': '',
            'home-0-played': '1',
            'home-0-points': '0',
            'home-0-mvp': '0',

            'home-1-number': '8',
            'home-1-id': '',
            'home-1-played': '1',
            'home-1-points': '0',
            'home-1-mvp': '0',

            'home-2-number': '10',
            'home-2-id': '',
            'home-2-played': '1',
            'home-2-points': '0',
            'home-2-mvp': '0',

            'home-3-number': '12',
            'home-3-id': '',
            'home-3-played': '1',
            'home-3-points': '0',
            'home-3-mvp': '0',

            'home-TOTAL_FORMS': '4',
            'home-INITIAL_FORMS': '4',
            'home-MAX_NUM_FORMS': '1000',

            'away-TOTAL_FORMS': '0',
            'away-INITIAL_FORMS': '0',
            'away-MAX_NUM_FORMS': '1000',
        }

        res = self.client.post(url, data=data)
        text = '''
        <ul class="list-group">
            <li class="list-group-item list-group-item-danger">
                Total number of points (0) does not equal total
                number of scores (3) for this team.
            </li>
        </ul>
        '''
        self.assertContains(res, text, html=True)

        data.update({
            'home-0-points': '0',
            'home-1-points': '1',
            'home-2-points': '0',
            'home-3-points': '2',
        })
        res = self.client.post(url, data=data)
        self.assertRedirects(res, reverse('admin:index'))

        # Make sure the database is correctly reflecting the detailed entries.
        self.assertEqual(m.statistics.count(), 4)

    def test_bug_0068(self):
        """
        While there are no LadderEntry or LadderSummary records for matches
        revealed in the ``match-results`` view, the submission worked fine.

        However on subsequent processing the method decorator is invoked
        differently, and there is now "raw" keyword argument - this resulted
        in issue #68.
        """
        match = factories.MatchFactory.create()
        data = {
            # first formset
            "matches-INITIAL_FORMS": "1",
            "matches-MAX_NUM_FORMS": "1000",
            "matches-TOTAL_FORMS": "1",

            "matches-0-id": str(match.pk),
            "matches-0-home_team_score": "4",
            "matches-0-away_team_score": "3",
            "matches-0-is_forfeit": "0",

            # second formset
            "byes-INITIAL_FORMS": "0",
            "byes-MAX_NUM_FORMS": "1000",
            "byes-TOTAL_FORMS": "0",
        }

        self.post('admin:fixja:match-results',
                  match.stage.division.season.competition.pk,
                  match.stage.division.season.pk,
                  match.date.strftime('%Y%m%d'),
                  data=data)
        self.response_302()

        data.update({
            "matches-1-home_team_score": "5",
            "matches-1-away_team_score": "2",
        })
        self.post('admin:fixja:match-results',
                  match.stage.division.season.competition.pk,
                  match.stage.division.season.pk,
                  match.date.strftime('%Y%m%d'),
                  data=data)
        self.response_302()

    def test_enhancement_0024_team(self):
        """
        When editing a Team or UndecidedTeam it should be possible to delete
        the object if there are no matches that team has been assigned to.
        """
        stage = factories.StageFactory.create()
        home = factories.TeamFactory.create(division=stage.division)
        away = factories.UndecidedTeamFactory.create(stage=stage)

        factories.MatchFactory.create(
            stage=stage,
            home_team=home,
            away_team=None, away_team_undecided=away)

        delete_url = self.reverse(
            home._get_admin_namespace() + ':delete', *home._get_url_args())

        # Ensure the edit view response does not include a delete button.
        self.get(home._get_admin_namespace() + ':edit', *home._get_url_args())
        self.assertResponseNotContains(delete_url, html=False)

        # Ensure that visiting the delete view does not respond when matches
        # are associated with the Team.
        self.get(
            home._get_admin_namespace() + ':delete', *home._get_url_args())
        self.response_410()

        # Ensure that POST to the delete view fails the same as for GET.
        self.post(
            home._get_admin_namespace() + ':delete', *home._get_url_args())
        self.response_410()

        # Create a new Team and re-check the scenarios.
        team = factories.TeamFactory.create(division=stage.division)

        delete_url = self.reverse(
            team._get_admin_namespace() + ':delete', *team._get_url_args())

        # Ensure the edit view response does not include a delete button.
        self.get(team._get_admin_namespace() + ':edit', *team._get_url_args())
        self.assertResponseNotContains(delete_url, html=False)

        # Ensure that visiting the delete view is not allowed when there are no
        # matches but you use a GET request.
        self.get(
            team._get_admin_namespace() + ':delete', *team._get_url_args())
        self.response_405()

        # Ensure that POST to the delete view redirects.
        self.post(
            team._get_admin_namespace() + ':delete', *team._get_url_args())
        self.response_302()

        # Subsequent GET request to the edit view should be a 404.
        self.get(
            team._get_admin_namespace() + ':edit', *team._get_url_args())
        self.response_404()

    def test_enhancement_0024_undecidedteam(self):
        """
        When editing a Team or UndecidedTeam it should be possible to delete
        the object if there are no matches that team has been assigned to.
        """
        stage = factories.StageFactory.create()
        home = factories.TeamFactory.create(division=stage.division)
        away = factories.UndecidedTeamFactory.create(stage=stage)

        factories.MatchFactory.create(stage=stage,
                                      home_team=home,
                                      away_team=None,
                                      away_team_undecided=away)

        delete_url = self.reverse(away._get_admin_namespace() + ':delete',
                                  *away._get_url_args())

        # Ensure the edit view response does not include a delete button.
        self.get(away._get_admin_namespace() + ':edit', *away._get_url_args())
        self.assertResponseNotContains(delete_url, html=False)

        # Ensure that visiting the delete view does not respond when matches
        # are associated with the Team.
        self.get(away._get_admin_namespace() + ':delete',
                 *away._get_url_args())
        self.response_410()

        # Ensure that POST to the delete view fails the same as for GET.
        self.post(away._get_admin_namespace() + ':delete',
                  *away._get_url_args())
        self.response_410()

        # Create a new UndecidedTeam and re-check the scenarios.
        team = factories.UndecidedTeamFactory.create(stage=stage)

        delete_url = self.reverse(team._get_admin_namespace() + ':delete',
                                  *team._get_url_args())

        # Ensure the edit view response *does* include a delete button.
        self.get(stage._get_admin_namespace() + ':edit',
                 *stage._get_url_args())
        self.assertResponseContains(delete_url, html=False)

        # Ensure that visiting the delete view is not allowed when there are no
        # matches but you use a GET request.
        self.get(team._get_admin_namespace() + ':delete',
                 *team._get_url_args())
        self.response_405()

        # Ensure that POST to the delete view redirects.
        self.post(team._get_admin_namespace() + ':delete',
                  *team._get_url_args())
        self.response_302()

        # Subsequent GET request to the edit view should be a 404.
        self.get(team._get_admin_namespace() + ':delete',
                 *team._get_url_args())
        self.response_404()

    def test_bug_80_add_stagegroup(self):
        "Add Pool should gain order equal to max(order) + 1"
        pool = factories.StageGroupFactory.create()
        data = {
            'title': 'Group Z',
            'short_title': '',
            'slug': '',
            'slug_locked': '0',
            'carry_ladder': '0',
        }
        self.post(pool._get_admin_namespace() + ':add',
                  *pool._get_url_args()[:-1], data=data)
        self.response_302()
        z = pool.stage.pools.latest('order')
        self.assertEqual(z.order, pool.order + 1)

    def test_bug_0115_add_duplicate_team_name(self):
        """
        When adding two teams to the same division, it should not allow the
        second one because of database integrity constraints.
        """
        team = factories.TeamFactory.create()
        data = {
            'title': team.title,
        }
        self.post(team._get_admin_namespace() + ':add',
                  *team._get_url_args()[:-1], data=data)
        self.assertFormError(
            self.last_response, 'form', 'title',
            ["Team with this Title already exists."])

    def test_team_add(self):
        division = factories.DivisionFactory.create()
        data = {
            'title': 'New Team',  # current value
            'short_title': '',
            'names_locked': '0',
            'timeslots_after': '',
            'timeslots_before': '',
            'team_clashes': [],
        }
        self.post(division._get_admin_namespace() + ':team:add',
                  *division._get_url_args(), data=data)
        self.response_302()
        # Ensure that the team was created and that it has a slug
        team = Team.objects.get(title='New Team')
        self.assertEqual(team.slug, 'new-team')

    def test_team_edit(self):
        team = factories.TeamFactory.create()
        data = {
            'title': 'Blue',
            'short_title': '',
            'names_locked': '0',
            'timeslots_after': '19:20',  # new value
            'timeslots_before': '',
            'team_clashes': [],
        }

        self.post(team._get_admin_namespace() + ':edit',
                  *team._get_url_args(), data=data)
        self.response_302()

        # Ensure that the team was updated
        blue = Team.objects.get(title='Blue')
        self.assertEqual(team.pk, blue.pk)
        self.assertEqual(blue.slug, 'blue')
        self.assertEqual(blue.timeslots_after, time(19, 20))

        # Try again, update the title and make sure the slug changes
        data['title'] = 'Magenta'
        self.post(blue._get_admin_namespace() + ':edit',
                  *blue._get_url_args(), data=data)
        self.response_302()

        magenta = Team.objects.get(title='Magenta')
        self.assertEqual(team.pk, magenta.pk)
        self.assertEqual(magenta.slug, 'magenta')
        self.assertEqual(magenta.timeslots_after, time(19, 20))

    def test_add_stage_duplicate_slug(self):
        """
        When adding a new stage to a division it should not be possible to add
        a second stage with a slug already used because of database integrity
        constraints.
        """
        stage = factories.StageFactory.create()
        data = {
            'title': stage.title,
        }
        self.post(stage._get_admin_namespace() + ':add',
                  *stage._get_url_args()[:-1], data=data)
        self.assertFormError(
            self.last_response, 'form', 'title',
            ["Stage with this Title already exists."])



"""
Django settings for vitriolic project.

Generated by 'django-admin startproject' using Django 1.10.3.

For more information on this file, see
https://docs.djangoproject.com/en/1.10/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.10/ref/settings/
"""

import os
import time

import django
import environ
from django.urls import reverse_lazy

env = environ.Env()

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

SILENCED_SYSTEM_CHECKS = env.list('SILENCED_SYSTEM_CHECKS', default=[])
SILENCED_SYSTEM_CHECKS.append('models.E006')

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.10/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = '3h_k7=3wv&i&#^36t=zv)l99bijpp06j((ld%7u7&3u)2!8iq8'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = []

SITE_ID = 1


# Disable the database migrations machinery to speed up test suite.
class DisableMigrations(object):
    """
    Trick learned at http://bit.ly/2vjVpNc
    """
    def __contains__(self, item):
        return True

    def __getitem__(self, item):
        return None if django.VERSION >= (1, 11) else 'notmigrations'


MIGRATION_MODULES = DisableMigrations()


# Application definition

INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.humanize',
    'django.contrib.messages',
    'django.contrib.postgres',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.staticfiles',

    'mptt',
    'cloudinary',
    'guardian',
    'bootstrap3',
    'django_gravatar',
    'embed_video',

    'touchtechnology.common',
    'touchtechnology.admin',
    'touchtechnology.content',
    'touchtechnology.news',

    'tournamentcontrol.competition',

    'example_app',
]

MIDDLEWARE = [
    'django.contrib.sites.middleware.CurrentSiteMiddleware',
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'touchtechnology.common.middleware.served_by_middleware',
    'touchtechnology.content.middleware.SitemapNodeMiddleware',
    'touchtechnology.content.middleware.redirect_middleware',
]

ROOT_URLCONF = 'vitriolic.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
                # It would be better if we could use modify_settings to append
                # these as required, but all the core Django checks look like
                # overriding TEMPLATES is done as override_settings and done
                # in entirety.
                'touchtechnology.common.context_processors.env',
                'touchtechnology.common.context_processors.query_string',
                'touchtechnology.common.context_processors.site',
                'touchtechnology.common.context_processors.tz',
            ],
        },
    },
]

WSGI_APPLICATION = 'vitriolic.wsgi.application'


# Database
# https://docs.djangoproject.com/en/1.10/ref/settings/#databases

DATABASES = {
    'default': env.db(default='psql://vitriolic:vitriolic@localhost/vitriolic'),
}

if DATABASES['default']['ENGINE'].startswith('django.db.backends.postgresql'):
    if os.getenv('POSTGRES_5432_TCP'):
        DATABASES['default']['PORT'] = env.int('POSTGRES_5432_TCP')
        # delay long enough to let the postgresql container startup
        time.sleep(4)


# Password validation
# https://docs.djangoproject.com/en/1.10/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]

AUTHENTICATION_BACKENDS = (
    'django.contrib.auth.backends.ModelBackend',  # this is default
    'guardian.backends.ObjectPermissionBackend',
)

LOGIN_URL = reverse_lazy('accounts:login')

ANONYMOUS_USER_NAME = 'anonymous'


# Internationalization
# https://docs.djangoproject.com/en/1.10/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.10/howto/static-files/

STATIC_URL = '/static/'


# Logging setup. Adjust handlers as required.

LOGGING = {
    "version": 1,
    "handlers": {
        "console": {"class": "logging.StreamHandler"},
        "null": {"class": "logging.NullHandler"},
    },
    "loggers": {
        "": {"level": "DEBUG", "handlers": ["null"]},
    },
}



from django.conf.urls import include, url
from example_app import sites, views
from touchtechnology.common.sites import AccountsSite
from touchtechnology.news.sites import NewsSite

test_context_processors = sites.TestContextProcessorsSite()
test_date_time_field = sites.TestDateTimeFieldSite()
test_generic_views = sites.TestGenericViewsSite()
test_query_string = sites.TestQueryStringSite()
test_pagination = sites.TestPaginationSite()

accounts = AccountsSite()
news = NewsSite()

urlpatterns = [
    url(r'^context-processors/', test_context_processors.urls),
    url(r'^date-time-field/', test_date_time_field.urls),
    url(r'^generic/', test_generic_views.urls),
    url(r'^query-string/', test_query_string.urls),
    url(r'^pagination/', test_pagination.urls),
    url(r'^accounts/', accounts.urls),  # can't login without it
    url(r'^news/', news.urls),
    url(r'^', include('touchtechnology.common.urls')),
    url(r'^$', views.index, name='index'),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-07-06 11:59
from __future__ import unicode_literals

import touchtechnology.common.db.models
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('news', '0003_category_last_modified'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='category',
            options={'verbose_name_plural': 'categories'},
        ),
        migrations.AlterField(
            model_name='article',
            name='is_active',
            field=touchtechnology.common.db.models.BooleanField(default=True, verbose_name='Enabled'),
        ),
        migrations.AlterField(
            model_name='article',
            name='last_modified',
            field=models.DateTimeField(auto_now=True),
        ),
        migrations.AlterField(
            model_name='article',
            name='published',
            field=models.DateTimeField(help_text='Set a date & time in the future to schedule an announcement.', verbose_name='Published'),
        ),
        migrations.AlterField(
            model_name='article',
            name='slug_locked',
            field=touchtechnology.common.db.models.BooleanField(default=False, verbose_name='Locked Slug'),
        ),
        migrations.AlterField(
            model_name='category',
            name='hidden_from_navigation',
            field=touchtechnology.common.db.models.BooleanField(default=False, help_text="When set to 'Yes' this object will still be available, but will not appear in menus.", verbose_name='Hide from menus'),
        ),
        migrations.AlterField(
            model_name='category',
            name='is_active',
            field=touchtechnology.common.db.models.BooleanField(default=True, verbose_name='Enabled'),
        ),
        migrations.AlterField(
            model_name='category',
            name='last_modified',
            field=models.DateTimeField(auto_now=True),
        ),
        migrations.AlterField(
            model_name='category',
            name='slug_locked',
            field=touchtechnology.common.db.models.BooleanField(default=False, verbose_name='Locked Slug'),
        ),
    ]



from __future__ import unicode_literals

from datetime import datetime
from os.path import dirname, join

from django.test.utils import modify_settings, override_settings
from django.utils import timezone
from django.utils.encoding import smart_str
from test_plus import TestCase
from touchtechnology.common.tests.test_timezone_forms import TestForm1


class TimeZoneTests(TestCase):

    maxDiff = None

    def get_html(self, filename):
        with open(join(dirname(__file__), 'test_html', filename)) as fp:
            html = fp.read()
        return smart_str(html)

    @override_settings(USE_TZ=False)
    def test_select_date_time_field_empty(self):
        data = {
            'timestamp_0': '4',
            'timestamp_1': '7',
            'timestamp_2': '2013',
            'timestamp_3': '',
            'timestamp_4': '',
        }

        # with only date fields completed validation should fail
        form = TestForm1(data=data)
        self.failIf(form.is_valid())
        self.assertEquals(form.errors['timestamp'],
                          ['Please enter a valid date and time.'])

        data.update({
            'timestamp_3': '7',
            'timestamp_4': '49',
        })

        # time fields added should allow validation to succeed
        form = TestForm1(data=data)
        self.failUnless(form.is_valid())

        # verify the cleaned value from user input is a datetime
        # object as we would expect it.
        timestamp = form.cleaned_data.get('timestamp')
        self.assertEqual(datetime(2013, 7, 4, 7, 49), timestamp)

    @override_settings(USE_TZ=True, TIME_ZONE='Australia/Sydney')
    def test_select_date_time_field_empty_tz(self):
        data = {
            'timestamp_0': '4',
            'timestamp_1': '7',
            'timestamp_2': '2013',
            'timestamp_3': '',
            'timestamp_4': '',
            'timestamp_5': 'Europe/London',
        }

        # with only date fields completed validation should fail
        form = TestForm1(data=data)
        self.failIf(form.is_valid())
        self.assertEquals(form.errors['timestamp'],
                          ['Please enter a valid date and time.'])

        data.update({
            'timestamp_3': '7',
            'timestamp_4': '49',
        })

        # time fields added should allow validation to succeed
        form = TestForm1(data=data)
        self.failUnless(form.is_valid())

        # verify the cleaned value from user input is a datetime
        # object as we would expect it.
        timestamp = form.cleaned_data.get('timestamp')
        self.assertEqual(
            datetime(2013, 7, 4, 16, 49),
            timezone.make_naive(timestamp, timezone.get_current_timezone()))

    @override_settings(USE_TZ=False)
    def test_select_date_time_field_initial(self):
        timestamp = datetime(2013, 3, 24, 14, 30)

        initial = {
            'timestamp': timestamp,
        }
        form = TestForm1(initial=initial)

        # check that the HTML output is as expected for a naked
        # form renderer.
        html1 = smart_str(form)
        html2 = self.get_html('select_date_time_field_initial.html')
        self.assertHTMLEqual(html1, html2)

    @override_settings(USE_TZ=True, TIME_ZONE='Australia/Sydney')
    def test_select_date_time_field_initial_tz(self):
        timestamp = timezone.make_aware(
            datetime(2013, 3, 24, 14, 30), timezone.get_current_timezone())

        initial = {
            'timestamp': timestamp,
        }
        form = TestForm1(initial=initial)

        # check that the HTML output is as expected for a naked
        # form renderer.
        html1 = smart_str(form)
        html2 = self.get_html('select_date_time_field_initial_tz.html')
        self.assertHTMLEqual(html1, html2)


@override_settings(ROOT_URLCONF='example_app.urls')
@modify_settings(MIDDLEWARE={
    'append': 'touchtechnology.common.middleware.timezone_middleware',
})
class TestTimezoneMiddleware(TestCase):

    def test_set_timezone(self):
        "Ensure time zone is correct before, during and after a set request"
        self.assertEqual(
            timezone.get_current_timezone_name(),
            timezone.get_default_timezone_name())

        data = {
            'timezone': 'Australia/Sydney',
        }
        self.post('set-timezone', data=data, follow=True)
        self.assertResponseContains(
            '<option value="Australia/Sydney" selected>Sydney</option>')

        self.assertEqual(
            timezone.get_current_timezone_name(),
            timezone.get_default_timezone_name())



from django.apps import AppConfig
from django.db.models.signals import post_migrate


def install_placeholder_handler(sender, **kwargs):
    from touchtechnology.content.utils import install_placeholder
    install_placeholder(sender.name)


class ContentConfig(AppConfig):
    name = "touchtechnology.content"

    def ready(self):
        post_migrate.connect(install_placeholder_handler)



import logging

from django.template import Library
from touchtechnology.content.models import Chunk

logger = logging.getLogger(__name__)
register = Library()


@register.simple_tag
def chunk(slug):
    instance, created = Chunk.objects.get_or_create(slug=slug)
    if created:
        logger.debug('Created %r' % instance)
    return instance.copy



import datetime
import functools
from operator import add, or_

from dateutil.parser import parse
from django import forms
from django.db.models import Q
from django.http import HttpResponse, HttpResponseRedirect
from django.utils.translation import ugettext_lazy as _
from first import first
from formtools.wizard.views import SessionWizardView
from six.moves import reduce
from touchtechnology.common.forms.fields import ModelChoiceField, ModelMultipleChoiceField
from tournamentcontrol.competition.models import Season
from tournamentcontrol.competition.tasks import generate_pdf_scorecards
from tournamentcontrol.competition.utils import generate_scorecards

##############################################################################
# Scorecard Report
##############################################################################


def combine_date_time_tuple(t):
    return datetime.datetime.combine(*t)


def q_date_from_datetime(dt):
    return Q(date=dt.date())


def q_date_time_from_datetime(dt):
    return Q(date=dt.date(), time=dt.time())


no_date_or_time = Q(date__isnull=True) | Q(time__isnull=True)


class SeasonForm(forms.Form):
    """
    Step 1. Select which season you want to generate a report of.
    """

    def __init__(self, *args, **kwargs):
        super(SeasonForm, self).__init__(*args, **kwargs)

        seasons = Season.objects.exclude(complete=True) \
                                .select_related('competition') \
                                .order_by('-start_date')
        self.fields['season'] = ModelChoiceField(
            queryset=seasons,
            label_from_instance=lambda s: "{0} {1}".format(s.competition.title,
                                                           s.title))


class FilterForm(forms.Form):
    """
    Step 2. Provide fields to allow the admin to filter the matches for
    inclusion in the report.
    """

    def __init__(self, season, *a, **kw):
        super(FilterForm, self).__init__(*a, **kw)

        # given a Season instance get all the matches which should have a
        # scorecard produced.
        self.matches = season.matches.exclude(no_date_or_time)

        # allow filtering by division
        self.fields['division'] = ModelMultipleChoiceField(
            queryset=season.divisions.all(),
            label_from_instance='title',
            required=False)

        # allow filtering by date
        dates = self.matches.order_by('date') \
                            .values_list('date', flat=True) \
                            .distinct()

        date_choices = zip(dates, dates)
        self.fields['dates'] = forms.MultipleChoiceField(
            choices=date_choices,
            widget=forms.CheckboxSelectMultiple,
            required=False)

        # allow filtering by timeslot
        timeslots = self.matches.order_by('date', 'time') \
                                .values_list('date', 'time') \
                                .distinct()

        datetimes = [combine_date_time_tuple(t) for t in timeslots]
        datetime_choices = zip(datetimes, datetimes)
        self.fields['timeslots'] = forms.MultipleChoiceField(
            choices=datetime_choices,
            widget=forms.CheckboxSelectMultiple,
            required=False)

        # allow configurable templates to be rendered
        template_choices = (
            ('', _('Choose a report template...')),
            ('scorecards.html', 'Scorecards'),
            ('signon.html', 'Sign-on Sheet'),
            ('mvp.html', 'Most Valuable Player'),
        )

        self.fields['template'] = forms.ChoiceField(
            choices=template_choices,
            label=_('Template'))

        output_choices = (
            ('', _('Choose an output format...')),
            ('pdf', _('PDF')),
            ('html', _('HTML')),
        )
        self.fields['format'] = forms.ChoiceField(
            choices=output_choices,
            label=_('Output as'))

    def clean(self):
        data = self.cleaned_data.copy()
        query_filter = Q()

        dates = self.cleaned_data.get('dates')
        timeslots = self.cleaned_data.get('timeslots')

        if dates:
            date_list = [
                q_date_from_datetime(parse(d))
                for d in dates
            ]
            query_filter |= functools.reduce(or_, date_list)

        if timeslots:
            timeslot_list = [
                q_date_time_from_datetime(parse(t))
                for t in timeslots
            ]
            query_filter |= functools.reduce(or_, timeslot_list)

        divisions = self.cleaned_data.get('division')

        if divisions:
            query_filter &= Q(stage__division__in=divisions)

        matches = self.matches.filter(query_filter)
        if not matches.count():
            raise forms.ValidationError(_("There are no matches that satisfy "
                                          "your criteria."))

        data['matches'] = matches

        return data


class ScorecardWizardBase(SessionWizardView):

    def get_context_data(self, form, **kwargs):
        context = super(
            ScorecardWizardBase, self).get_context_data(form, **kwargs)
        context.update(self.extra_context)
        return context

    def get_form_kwargs(self, step):
        # pass the cleaned_data from the previous step into the current step
        return self.get_cleaned_data_for_step(
            self.get_prev_step(step=step)) or {}

    def get_template_names(self):
        return self.app.template_path('scorecards/report.html')

    def done(self, form_list, **kwargs):
        data = self.get_all_cleaned_data()
        format = data.get('format')
        season = data.pop('season')
        matches = data.pop('matches')
        template = data.pop('template')

        extra_context = {
            'competition': season.competition,
            'season': season,
        }

        templates = self.app.template_path(
            template, season.competition.slug, season.slug)

        if format == 'pdf':
            kw = {
                'matches': matches,
                'templates': templates,
                'extra_context': extra_context,
            }
            if hasattr(self.request, 'tenant'):
                kw['_schema_name'] = self.request.tenant.schema_name
                kw['base_url'] = self.request.build_absolute_uri('/')

            result = generate_pdf_scorecards.delay(**kw)

            reverse_kwargs = {
                'competition_id': season.competition_id,
                'season_id': season.pk,
                'result_id': result.id,
            }

            redirect_to = self.app.reverse('competition:season:scorecards-async',
                                           kwargs=reverse_kwargs)
            response = HttpResponseRedirect(redirect_to)
        else:
            output = generate_scorecards(matches, templates, extra_context)
            response = HttpResponse(output)

        return response


def scorecardwizard_factory(**kwargs):
    return type('ScorecardWizard', (ScorecardWizardBase,), kwargs)


##############################################################################
# Draw Generation
##############################################################################

class DrawGenerationWizard(SessionWizardView):

    extra_context = {}
    redirect_to = None
    stage = None
    template_name = 'tournamentcontrol/competition/admin/draw_generation/wizard.html'

    def get_form_initial(self, step):
        if step == '0':
            if self.stage.pools.count():
                queryset = self.stage.pools.all()
            else:
                queryset = self.stage.division.stages.filter(pk=self.stage.pk)
            return queryset

    def get_form_kwargs(self, step):
        if step == '1':
            data = self.get_cleaned_data_for_step('0')
            matches = functools.reduce(add, [d['matches'] for d in data])
            return {'queryset': matches}
        return {}

    def get_context_data(self, form, **kwargs):
        context = super(DrawGenerationWizard, self).get_context_data(
            form=form, **kwargs)
        context.update(self.extra_context)
        context.update({
            'model': self.stage.matches.none(),
        })
        return context

    def done(self, form_list, **kwargs):
        first(reversed(form_list)).save()
        return HttpResponseRedirect(self.redirect_to)



from django.conf.urls import include, url

urlpatterns = [
    url(r"v1/", include("tournamentcontrol.competition.rest.v1.urls", namespace="v1")),
    url(r"^api-auth/", include("rest_framework.urls")),
]



from setuptools import find_packages, setup

try:
    with open('README.md') as fp:
        long_description = fp.read()
except IOError:
    long_description = None

setup(
    name='vitriolic',
    version='1.17.0',
    author='Gary Reynolds',
    author_email='gary@touch.asn.au',
    license='BSD',
    description=(
        'A suite of packages for the Django framework to provide integrated '
        'Content Management and Sports Administration functionality for '
        'sporting associations.'
    ),
    long_description=long_description,
    packages=find_packages(),
    install_requires=[
        'pillow',
        'django-classy-tags~=0.7.2',
        'django-froala-editor>=2.7.1',
        'django-guardian<2',
        'django-modelforms~=0.2,!=0.2.0',
        'django-mptt>=0.9,<0.10',
        'django>=1.11,<2.1',
        'first~=2.0.1',
        'namedentities',
        'psycopg2-binary',
        'python-dateutil~=2.5.3',
        'pytz',
    ],
    extras_require={
        'test': [
            'coverage',
            'coveralls[yaml]',
            'django-environ',
            'django-test-plus>=1.0.15,<1.0.22',
            'factory_boy',
            'freezegun',
            'mock',
        ],
        'redis': [
            'django-redis-cache>=1.8.0,<2',
        ],
        'admin': [
            'django-bootstrap3>=8.2,<9',
            'django-gravatar2>=1.4,<2',
        ],
        'content': [
        ],
        'news': [
            'django-imagekit',
            'python-magic',
        ],
        'competition': [
            'backports.statistics',
            'cloudinary',
            'django-celery>=3.1.0',
            'django-embed-video<1.3',
            'django-formtools>=2.1',
            'django-mathfilters',
            'djangorestframework',
            'drf-nested-routers',
            'icalendar>=3.9.0,<4',
            'pyparsing>=2.0.3,<3',
        ],
    },
    include_package_data=True,
    zip_safe=False,
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import django.db.models.deletion
import touchtechnology.common.db.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('competition', '0013_statistic_played_field'),
    ]

    operations = [
        migrations.AlterField(
            model_name='match',
            name='play_at',
            field=touchtechnology.common.db.models.ForeignKey(on_delete=django.db.models.deletion.SET_NULL, blank=True, to='competition.Place', null=True),
        ),
    ]



try:
    from urllib.parse import urljoin
except ImportError:
    from urlparse import urljoin

from django.conf.urls import include, url
from django.utils.translation import ugettext_lazy as _
from touchtechnology.admin.base import AdminComponent
from touchtechnology.admin.sites import site
from touchtechnology.common.decorators import staff_login_required_m
from touchtechnology.news.forms import (
    ArticleContentFormset, ArticleForm, CategoryForm,
)
from touchtechnology.news.models import Article, ArticleContent, Category


class NewsAdminComponent(AdminComponent):
    verbose_name = _('News')
    unprotected = False

    def __init__(self, app, name="news", app_name="news"):
        super(NewsAdminComponent, self).__init__(app, name, app_name)

    def get_urls(self):
        article_patterns = ([
            url(r'^$', self.list_articles, name='list'),
            url(r'^add/$', self.edit_article, name='add'),
            url(r'^(?P<pk>\d+)/$', self.edit_article, name='edit'),
            url(r'^(?P<pk>\d+)/delete/$',
                self.delete_article, name='delete'),
            url(r'^(?P<pk>\d+)/permission/$',
                self.perms_article, name='perms'),
        ], self.app_name)

        category_patterns = ([
            url(r'^$', self.list_categories, name='list'),
            url(r'^add/$', self.edit_category, name='add'),
            url(r'^(?P<pk>\d+)/$', self.edit_category, name='edit'),
            url(r'^(?P<pk>\d+)/delete/$',
                self.delete_category, name='delete'),
            url(r'^(?P<pk>\d+)/permission/$',
                self.perms_category, name='perms'),
        ], self.app_name)

        urlpatterns = [
            url(r'^$', self.index, name='index'),
            url(r'^article/', include(article_patterns, namespace='article')),
            url(r'^category/', include(category_patterns, namespace='category')),
        ]
        return urlpatterns

    def dropdowns(self):
        dl = (
            (_('Articles'), self.reverse('article:list'), 'newspaper-o'),
            (_('Categories'), self.reverse('category:list'), 'tag'),
        )
        return dl

    @staff_login_required_m
    def index(self, request, *args, **kwargs):
        return self.redirect(self.reverse('article:list'))

    # Article views

    @staff_login_required_m
    def list_articles(self, request, *args, **kwargs):
        return self.generic_list(request, Article,
                                 paginate_by=25,
                                 permission_required=True,
                                 extra_context=kwargs)

    @staff_login_required_m
    def edit_article(self, request, pk=None, *args, **kwargs):
        return self.generic_edit_related(
            request, Article, ArticleContent,
            pk=pk,
            form_class=ArticleForm,
            form_kwargs={'user': request.user},
            formset_class=ArticleContentFormset,
            # permission_required=True,
            post_save_redirect=self.redirect(urljoin(request.path, '..')),
            extra_context=kwargs)

    @staff_login_required_m
    def delete_article(self, request, pk, **kwargs):
        return self.generic_delete(request, Article, pk=pk,
                                   permission_required=True)

    @staff_login_required_m
    def perms_article(self, request, pk, **extra_context):
        return self.generic_permissions(
            request, Article, pk=pk, **extra_context)

    # Category views

    @staff_login_required_m
    def list_categories(self, request, *args, **kwargs):
        return self.generic_list(request, Category,
                                 paginate_by=25,
                                 permission_required=True,
                                 extra_context=kwargs)

    @staff_login_required_m
    def edit_category(self, request, pk=None, *args, **kwargs):
        return self.generic_edit(request, Category, pk=pk,
                                 form_class=CategoryForm,
                                 form_kwargs={'user': request.user},
                                 permission_required=True,
                                 post_save_redirect=self.redirect(
                                     urljoin(request.path, '..')),
                                 extra_context=kwargs)

    @staff_login_required_m
    def delete_category(self, request, pk, **kwargs):
        return self.generic_delete(request, Category, pk=pk,
                                   permission_required=True)

    @staff_login_required_m
    def perms_category(self, request, pk, **extra_context):
        return self.generic_permissions(
            request, Category, pk=pk,
            post_save_redirect=self.redirect(
                self.reverse('category:edit', kwargs={'pk': pk})),
            **extra_context)

site.register(NewsAdminComponent)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-07-24 22:25
from __future__ import unicode_literals

import uuid

import cloudinary.models
import django.db.models.deletion
from django.db import migrations, models
from django.utils.translation import ugettext_lazy as _


class Migration(migrations.Migration):

    dependencies = [("competition", "0032_resave_all_matches")]

    operations = [
        migrations.CreateModel(
            name="MatchScoreSheet",
            fields=[
                (
                    "uuid",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "image",
                    cloudinary.models.CloudinaryField(
                        max_length=255, verbose_name=_("Image")
                    ),
                ),
                (
                    "match",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="competition.Match",
                        related_name='scoresheets',
                    ),
                ),
            ],
        )
    ]



from __future__ import unicode_literals

import functools
import logging
from operator import attrgetter, or_

import guardian.shortcuts
import pytz
from django.apps import apps
from django.conf import settings
from django.contrib.auth import REDIRECT_FIELD_NAME
from django.contrib.auth.views import redirect_to_login
from django.core.cache import cache
from django.core.exceptions import PermissionDenied
from django.core.paginator import Paginator
from django.db import connection
from django.db.models import Model, Q
from django.http import Http404, HttpRequest, HttpResponseForbidden
from django.shortcuts import render_to_response
from django.template import RequestContext, TemplateDoesNotExist
from django.template.loader import select_template
from first import first
from guardian.conf import settings as guardian_settings
from six.moves import reduce
from touchtechnology.common.exceptions import NotModelManager
from touchtechnology.common.mixins import NodeRelationMixin

logger = logging.getLogger(__name__)


class FauxNodeMeta(object):

    left_attr = 'lft'
    right_attr = 'rght'
    level_attr = 'level'


class FauxNode(NodeRelationMixin):

    REPR_FMT = '<{class}: {title} ({tree_id}:{lft},{rght})>'

    def __init__(self, app, title, short_title, slug, level, tree_id,
                 lft=None, rght=None, parent=None,
                 changefreq='monthly', priority=0.5):
        self.app = app
        self.title = title
        self.short_title = short_title
        self.slug = slug
        self.level = level
        self.tree_id = tree_id
        self.lft = lft
        self.rght = rght
        if parent is None and app is not None:
            self.parent = app.node
        else:
            self.parent = parent

        try:
            self.changefreq = changefreq
        except AttributeError:
            logger.exception('Unable to set `changefreq` for %r', self)

        try:
            self.priority = priority
        except AttributeError:
            logger.exception('Unable to set `priority` for %r', self)

        self._mptt_meta = FauxNodeMeta()

    def __eq__(self, other):
        if other is None:
            return
        attr = ('title', 'short_title', 'slug', 'level', 'tree_id', 'parent')
        gttr = [attrgetter(a) for a in attr]
        return all([g(self) == g(other) for g in gttr])

    def __repr__(self):
        attr = ('title', 'short_title', 'slug', 'level', 'tree_id', 'lft', 'rght', 'parent')
        gttr = [attrgetter(a) for a in attr]
        data = dict(zip(attr, [g(self) for g in gttr]))
        data['class'] = self.__class__.__name__
        return self.REPR_FMT.format(**data)

    @classmethod
    def equal(cls, iterable, find):
        return first(iterable, key=lambda i: i == find)

    def get_absolute_url(self):
        raise NotImplementedError

    def get_ancestors(self):
        n = self
        while n.parent:
            yield n.parent
            n = n.parent

    def get_root(self):
        if self.parent is None:
            return self
        return self.parent.get_root()


def get_mod_func(callback):
    # Converts 'touchtechnology.common.utils.get_mod_func' to
    # ['touchtechnology.common.utils', 'get_mod_func']
    try:
        dot = callback.rindex('.')
    except ValueError:
        return callback, ''
    return callback[:dot], callback[dot + 1:]


def create_exclude_filter(queryset):
    def _filter(node):
        return Q(tree_id=node.tree_id, lft__gte=node.lft, lft__lte=node.rght)
    return functools.reduce(or_, [[x for x in n if _filter(x)] for n in queryset], Q())


def get_403_or_None(request, perms, obj=None, login_url=None,
                    redirect_field_name=None, return_403=False,
                    accept_global_perms=False, any_perm=False):
    """
    Copied from django-guardian (v1.2.5) and added the ability to decide if we
    are willing to accept ANY of the listed perms or they must ALL be included
    (the default).
    """
    login_url = login_url or settings.LOGIN_URL
    redirect_field_name = redirect_field_name or REDIRECT_FIELD_NAME

    # Handles both original and with object provided permission check
    # as ``obj`` defaults to None
    check = any if any_perm else all

    has_permissions = False

    if accept_global_perms:
        has_permissions = check([
            request.user.has_perm(perm)
            for perm in perms
        ])

    if not has_permissions:
        has_permissions = check([
            request.user.has_perm(perm, obj)
            for perm in perms
        ])

    if not has_permissions:
        if return_403:
            if guardian_settings.RENDER_403:
                try:
                    response = render_to_response(
                        guardian_settings.TEMPLATE_403, {},
                        RequestContext(request))
                    response.status_code = 403
                    return response
                except TemplateDoesNotExist as e:
                    if settings.DEBUG:
                        raise e
            elif guardian_settings.RAISE_403:
                raise PermissionDenied
            return HttpResponseForbidden()
        else:
            return redirect_to_login(request.get_full_path(),
                                     login_url,
                                     redirect_field_name)


def get_objects_for_user(user, perms, model_or_manager, use_groups=True,
                         any_perm=False, with_superuser=True):
    """
    The get_objects_for_user shortcut does not take into account any globally
    granted permissions, we want to check these first and then defer decision
    to django-guardian only if required.
    """
    model, manager = model_and_manager(model_or_manager)

    check = any if any_perm else all
    if check([user.has_perm(p) for p in perms]):
        return manager.all()

    return guardian.shortcuts.get_objects_for_user(
        user, perms, klass=model, any_perm=True)


def get_perms_for_model(model, add=False, change=False, delete=False, *extra):
    tpl = '%s.%%s_%s' % (model._meta.app_label, model._meta.model_name)
    res = []
    if add:
        res.append(tpl % 'add')
    if change:
        res.append(tpl % 'change')
    if delete:
        res.append(tpl % 'delete')
    for x in extra:
        res.append(tpl % x)
    return res


def get_all_perms_for_model(model, *extra):
    return get_perms_for_model(model, True, True, True, *extra)


def tree_for_node(node):
    """
    Utility function to construct a QuerySet which has the expanded set
    of nodes for the given node.
    """
    from .models import SitemapNode

    if not isinstance(node, SitemapNode):
        real_nodes = SitemapNode.objects.filter(
            tree_id=node.tree_id, lft__lte=node.lft)
        node = first(real_nodes.reverse())

    if node:
        parents = node.get_ancestors()
        siblings = node.get_siblings(include_self=True)
        children = node.get_descendants()

        rest = functools.reduce(
            or_, [p.get_siblings() for p in parents], SitemapNode.objects.none()
        )

        nodes = parents | siblings | children | rest
        nodes = nodes.filter(level__lte=node.level + 1)

    else:
        nodes = SitemapNode.objects.none()

    return nodes


def model_and_manager(model_or_manager):
    """
    All our generic views work on the basis of a ``model_or_manager`` which
    needs to be unpacked into a ``model`` and ``manager``.

    The function will reduce boilerplate in generic methods.
    """
    if isinstance(model_or_manager, Model):
        model = model_or_manager._meta.model
        manager = model._default_manager
    elif hasattr(model_or_manager, 'model'):
        model = model_or_manager.model
        manager = model_or_manager
    else:
        try:
            model = model_or_manager
            manager = model_or_manager._default_manager
        except AttributeError:
            raise NotModelManager(model_or_manager)
    return model, manager


def determine_page_number(request, paginator):
    """
    Determine what page number for the current request.

        request:    django.http.HttpRequest instance
        paginator:  django.core.paginator.Paginator instance
    """
    assert isinstance(request, HttpRequest)
    assert isinstance(paginator, Paginator)
    page = request.GET.get('page', 1)
    try:
        page_number = int(page)
    except ValueError:
        if page == 'last':
            page_number = paginator.num_pages
        else:
            raise Http404
    return page_number


def select_template_name(templates):
    """
    Return the template name that wins template resolution for an iterable of
    template names.
    """
    try:
        # Django 1.8+
        template = select_template(templates)
        template_name = template.template.name
    except AttributeError:
        # Django 1.7
        template_name = template.name
    return template_name


def get_timezone_from_request(request):
    if hasattr(request, 'session'):
        tzname = request.session.get('django_timezone', None)
        if tzname is not None:
            try:
                tzinfo = pytz.timezone(tzname)
            except pytz.UnknownTimeZoneError:
                tzinfo = None
            return tzinfo


def get_base_url(scheme='http'):
    """
    It is frustratingly difficult to obtain the appropriate base URL to use
    for sending links outside of the request-response cycle. If we are using
    the django-tenant-schemas application, this can be somewhat simplified.
    If we are not, fall back to using the django.contrib.sites framework.
    """
    if hasattr(connection, 'tenant'):
        hostname = connection.tenant.domain_url
        # We have our own pattern of using the `prepend_www` attribute so lets
        # check for it and stick it on the front.
        if getattr(connection.tenant, 'prepend_www', False):
            hostname = 'www.' + hostname
    else:
        site = apps.get_model('sites.Site').objects.get_current()
        hostname = site.domain
    context = {
        'scheme': scheme,
        'hostname': hostname,
    }
    return '%(scheme)s://%(hostname)s/' % context


def get_all_perms_for_model_cached(model, ttl=60, **extra):
    """
    Private function to cache and return the list of permissions that can be
    set on a given model.
    """
    # What are the permissions that this model accepts?
    cache_key = 'model_perms.%s.%s' % (
        model._meta.app_label, model._meta.model_name)
    model_perms = cache.get(cache_key)
    logger.debug(
        'model_perms_cache=%s', 'MISS' if model_perms is None else 'HIT')
    if model_perms is None:
        model_perms = get_all_perms_for_model(model, **extra)
        cache.set(cache_key, model_perms, timeout=ttl)
    return model_perms



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations


def set_draft_false(apps, schema_editor):
    Division = apps.get_model("competition", "Division")
    Division.objects.update(draft=False)


class Migration(migrations.Migration):

    dependencies = [
        ('competition', '0002_division_draft'),
    ]

    operations = [
        migrations.RunPython(set_draft_false, set_draft_false),
    ]



from test_plus import TestCase
from touchtechnology.common.tests.factories import UserFactory


class AuthenticationTest(TestCase):

    def test_admin_login(self):
        self.get('admin:index')
        self.response_302()

    def test_admin_logout(self):
        user = UserFactory.create(is_staff=True)
        with self.login(user):
            self.get('accounts:logout')
            self.assertResponseContains(
                "<p>You have successfully been logged out.</p>")
